<?php

/**
 * Fired during plugin activation
 *
 * @link  https://pagevisitcounter.com
 * @since 3.0.1
 *
 * @package    Advanced_Visit_Counter
 * @subpackage Advanced_Visit_Counter/includes
 */

/**
 * Fired during plugin activation.
 *
 * This class defines all code necessary to run during the plugin's activation.
 *
 * @since      3.0.1
 * @package    Advanced_Visit_Counter
 * @subpackage Advanced_Visit_Counter/includes
 * @author     Ankit Panchal <wptoolsdev@gmail.com>
 */
class Advanced_Visit_Counter_Activator {


	/**
	 * Short Description. (use period)
	 *
	 * Long Description.
	 *
	 * @since 3.0.1
	 */
	public static function activate() {
		global $wpdb;

		set_transient( 'apvcSetPromo', 'no', 86400 );
		update_option( 'apvc_promo', 'yes' );
		$version       = get_option( 'apvc_version' );
		$history_table = $wpdb->prefix . 'avc_page_visit_history';

		$exSettings = get_option( 'apvc_configurations' );

		if ( is_countable( $exSettings ) && count( $exSettings ) == 0 || empty( $exSettings ) ) {
			$apvc_config = array();

			$apvc_config['apvc_post_types']                = array( 'post', 'page' );
			$apvc_config['apvc_ip_address']                = array();
			$apvc_config['apvc_exclude_counts']            = array();
			$apvc_config['apvc_exclude_users']             = array();
			$apvc_config['apvc_exclude_show_counter']      = array();
			$apvc_config['apvc_spam_controller']           = '';
			$apvc_config['apvc_show_conter_on_front_side'] = array( 'disable' );
			$apvc_config['apvc_default_text_color']        = array( '#000000' );
			$apvc_config['apvc_default_label']             = array( 'Visits: ' );
			$apvc_config['apvc_todays_label']              = array( 'Today: ' );
			$apvc_config['apvc_global_label']              = array( 'Total: ' );
			$apvc_config['apvc_default_border_radius']     = array( 0 );
			$apvc_config['apvc_default_background_color']  = array( '#ffffff' );
			$apvc_config['apvc_default_border_color']      = array( '#000000' );
			$apvc_config['apvc_default_border_width']      = array( 2 );
			$apvc_config['apvc_wid_alignment']             = array( 'center' );
			$apvc_config['apvc_show_today_count']          = '';
			$apvc_config['apvc_show_global_count']         = '';
			$apvc_config['apvc_widget_width']              = array( 300 );
			$apvc_config['apvc_atc_page_count']            = array( 'on' );

			update_option( 'apvc_configurations', $apvc_config );
		}

		include_once ABSPATH . 'wp-admin/includes/upgrade.php';

		if ( $wpdb->get_var( "SHOW TABLES LIKE '$history_table'" ) != $history_table ) {
			$sql = "CREATE TABLE $history_table (
				id int(11) unsigned NOT NULL AUTO_INCREMENT,
				article_id int(11) NOT NULL,
				article_type text NOT NULL,
				user_type text NOT NULL,
				device_type text NOT NULL,
				date  datetime NOT NULL,
				last_date  datetime NOT NULL,
				ip_address varchar(255) NOT NULL,
				browser_full_name varchar(255) NOT NULL,
				browser_short_name varchar(255) NOT NULL,
				browser_version varchar(255) NOT NULL,
				operating_system varchar(255) NOT NULL,
				http_referer varchar(255) NOT NULL,
				user_id int(9) NOT NULL,
				site_id int(9) NOT NULL,
				flag int(1) NULL,
				country varchar(255),
				PRIMARY KEY  (id)
			);";

			include_once ABSPATH . 'wp-admin/includes/upgrade.php';
			dbDelta( $sql );
		}

		if ( apvc_fs()->is__premium_only() ) {

			$smart_notifications = $wpdb->prefix . 'apvc_smart_notifications';
			if ( $wpdb->get_var( "SHOW TABLES LIKE '$smart_notifications'" ) != $smart_notifications ) {
				$sql = "CREATE TABLE $smart_notifications (
					id int(11) unsigned NOT NULL AUTO_INCREMENT,
					type text NOT NULL,
					ex_type text NOT NULL,
					recipients text NOT NULL,
					subject text NOT NULL,
					email_body text NOT NULL,
					PRIMARY KEY  (id)
				);";

				include_once ABSPATH . 'wp-admin/includes/upgrade.php';
				dbDelta( $sql );
			}

			$realtime_users = $wpdb->prefix . 'apvc_realtime_users';
			if ( $wpdb->get_var( "SHOW TABLES LIKE '$realtime_users'" ) != $realtime_users ) {
				$sql = "CREATE TABLE $realtime_users (
					id int(11) unsigned NOT NULL AUTO_INCREMENT,
					ref_id int(11) NOT NULL,
					date datetime NOT NULL,
					time time NOT NULL,
					ip_address text NOT NULL,
					country text NOT NULL,
					PRIMARY KEY  (id)
				);";

				include_once ABSPATH . 'wp-admin/includes/upgrade.php';
				dbDelta( $sql );
			}

			$user_locations = $wpdb->prefix . 'apvc_user_locations';
			if ( $wpdb->get_var( "SHOW TABLES LIKE '$user_locations'" ) != $user_locations ) {
				$sql = "CREATE TABLE $user_locations (
					id int(11) unsigned NOT NULL AUTO_INCREMENT,
					ent_id int(11) NOT NULL,
					city text,
					state text,
					country text,
					country_code text,
					continent text,
					continent_code text,
					PRIMARY KEY  (id)
				);";

				include_once ABSPATH . 'wp-admin/includes/upgrade.php';
				dbDelta( $sql );
			}

			$apvc_shortcodes = $wpdb->prefix . 'apvc_shortcodes';
			if ( $wpdb->get_var( "SHOW TABLES LIKE '$apvc_shortcodes'" ) != $apvc_shortcodes ) {
				$sql = "CREATE TABLE $apvc_shortcodes (
					id int(11) unsigned NOT NULL AUTO_INCREMENT,
					title text NOT NULL,
					shortcode text NOT NULL,
					config text NOT NULL,
					date datetime NOT NULL,
					PRIMARY KEY  (id)
				);";

				include_once ABSPATH . 'wp-admin/includes/upgrade.php';
				dbDelta( $sql );
			}

			$export_files = $wpdb->prefix . 'apvc_exported_data';
			if ( $wpdb->get_var( "SHOW TABLES LIKE '$export_files'" ) != $export_files ) {
				$sql = "CREATE TABLE $export_files (
					id int(11) unsigned NOT NULL AUTO_INCREMENT,
					title text NOT NULL,
					file_name text NOT NULL,
					file_location text NOT NULL,
					date datetime NOT NULL,
					PRIMARY KEY  (id)
				);";
				include_once ABSPATH . 'wp-admin/includes/upgrade.php';
				dbDelta( $sql );
			}
			$uploads     = wp_upload_dir();
			$upload_path = $uploads['basedir'] . '/apvc_export';
			mkdir( $upload_path, 0755 );
			$import_path = $uploads['basedir'] . '/apvc_import';
			mkdir( $import_path, 0755 );

			chmod( $upload_path, 0755 );
			chmod( $import_path, 0755 );
		}

		update_option( 'apvc_version', ADVANCED_PAGE_VISIT_COUNTER );
		delete_option( 'apvc_newsletter' );
		delete_option( 'avc_config' );
		update_option( 'apvc_notice', 'no' );
	}

}


