<?php

/**
 * All Database Queries are here.
 *
 * @link       https://pagevisitcounter.com
 * @since      4.0.0
 *
 * @package    Advanced_Visit_Counter
 * @subpackage Advanced_Visit_Counter/includes
 */

/**
 * Fired during plugin activation.
 *
 * This class defines all code necessary to run during the plugin's activation.
 *
 * @since      3.0.1
 * @package    Advanced_Visit_Counter
 * @subpackage Advanced_Visit_Counter/includes
 * @author     Ankit Panchal <wptoolsdev@gmail.com>
 */
class Advanced_Visit_Counter_Queries {

	public $transients = array( 'apvc_yearly_data', 'apvc_monthly_data', 'apvc_weekly_data', 'apvc_daily_data', 'apvc_browser_traffic_stats_data', 'apvc_browser_traffic_data', 'apvc_ref_traffic_data', 'apvc_os_data', 'apvc_orders_total', 'apvc_total_orders_data', 'apvc_total_products_sales', 'apvc_mvp_month_data', 'apvc_mvp_daily_data', 'apvc_fv_30_td', 'apvc_fv_30_past', 'apvc_fv_td', 'apvc_fv_past', 'apvc_get_visitors_mn_data', 'apvc_get_visitors_data', 'apvc_pvp_30_data', 'apvc_pvp_ip_30_data', 'apvc_pvp_daily_data', 'apvc_pvp_ip_daily_data', 'apvc_get_visit_stats' );

	public function apvc_number_format( $num ) {

		$op = get_option( 'numbers_in_k' );

		if ( $op == 'Yes' ) {

			if ( $num > 1000 ) {

				$x               = round( $num );
				$x_number_format = number_format( $x );
				$x_array         = explode( ',', $x_number_format );
				$x_parts         = [ 'k', 'm', 'b', 't' ];
				$x_count_parts   = count( $x_array ) - 1;
				$x_display       = $x;
				$x_display       = $x_array[0] . ( (int) $x_array[1][0] !== 0 ? '.' . $x_array[1][0] : '' );
				$x_display       .= $x_parts[ $x_count_parts - 1 ];

				return $x_display;

			}
		}

		return $num;

	}


	/**
	 * Advanced Page Visit Counter Get total counts of the last year.
	 *
	 * @since    3.0.1
	 */
	public function get_total_counts_of_the_year() {
		global $wpdb;
		$table       = APVC_DATA_TABLE;
		$currentDate = date( 'Y-m-d H:i:s', strtotime( date( 'Y-m-d 0:0:0' ) . ' -1 year' ) );

		$monthWise = $wpdb->get_results( "SELECT COUNT(*) as total_count, MONTH(date) as month, YEAR(date) as year FROM {$table} WHERE date >= '$currentDate' GROUP BY MONTH(date) order by year ASC", ARRAY_N );

		$totalCounts = $wpdb->get_var( "SELECT COUNT(*) as total_count FROM {$table} WHERE date >= '" . $currentDate . "'" );

		$monthArray = [];
		foreach ( $monthWise as $value ): array_push( $monthArray, $value[0] ); endforeach;

		return wp_json_encode( [
			"months_wise"  => $monthArray,
			"total_counts" => $totalCounts,
		] );
	}


	/**
	 * Advanced Page Visit Counter Get Browsers Logos to list on reports pages.
	 *
	 * @since    3.0.1
	 */
	public function get_browsers_logos() {
		$logos = [
			'Firefox' => 'firefox.png',
			'Chrome'  => 'chrome.png',
			'Safari'  => 'Safari.png',
			'MSIE'    => 'Internet-Explorer.png',
			'Opera'   => 'opera.png',
			'Vivaldi' => 'vivaldi.png',
			'default' => 'default-browser.png',
		];

		return wp_json_encode( $logos );
	}

	/**
	 * Advanced Page Visit Counter Get Country names to list on reports pages.
	 *
	 * @since    3.0.1
	 */
	public function get_country_names() {

		$allCountries = [
			"Afghanistan"                                  => "AF",
			"land Islands"                                 => "AX",
			"Albania"                                      => "AL",
			"Algeria"                                      => "DZ",
			"American Samoa"                               => "AS",
			"AndorrA"                                      => "AD",
			"Angola"                                       => "AO",
			"Anguilla"                                     => "AI",
			"Antarctica"                                   => "AQ",
			"Antigua and Barbuda"                          => "AG",
			"Argentina"                                    => "AR",
			"Armenia"                                      => "AM",
			"Aruba"                                        => "AW",
			"Australia"                                    => "AU",
			"Austria"                                      => "AT",
			"Azerbaijan"                                   => "AZ",
			"Bahamas"                                      => "BS",
			"Bahrain"                                      => "BH",
			"Bangladesh"                                   => "BD",
			"Barbados"                                     => "BB",
			"Belarus"                                      => "BY",
			"Belgium"                                      => "BE",
			"Belize"                                       => "BZ",
			"Benin"                                        => "BJ",
			"Bermuda"                                      => "BM",
			"Bhutan"                                       => "BT",
			"Bolivia"                                      => "BO",
			"Bosnia and Herzegovina"                       => "BA",
			"Botswana"                                     => "BW",
			"Bouvet Island"                                => "BV",
			"Brazil"                                       => "BR",
			"British Indian Ocean Territory"               => "IO",
			"Brunei Darussalam"                            => "BN",
			"Bulgaria"                                     => "BG",
			"Burkina Faso"                                 => "BF",
			"Burundi"                                      => "BI",
			"Cambodia"                                     => "KH",
			"Cameroon"                                     => "CM",
			"Canada"                                       => "CA",
			"Cape Verde"                                   => "CV",
			"Cayman Islands"                               => "KY",
			"Central African Republic"                     => "CF",
			"Chad"                                         => "TD",
			"Chile"                                        => "CL",
			"China"                                        => "CN",
			"Christmas Island"                             => "CX",
			"Cocos (Keeling) Islands"                      => "CC",
			"Colombia"                                     => "CO",
			"Comoros"                                      => "KM",
			"Congo"                                        => "CG",
			"Congo, The Democratic Republic of the"        => "CD",
			"Cook Islands"                                 => "CK",
			"Costa Rica"                                   => "CR",
			"Cote D'Ivoire"                                => "CI",
			"Croatia"                                      => "HR",
			"Cuba"                                         => "CU",
			"Cyprus"                                       => "CY",
			"Czechia"                                      => "CZ",
			"Denmark"                                      => "DK",
			"Djibouti"                                     => "DJ",
			"Dominica"                                     => "DM",
			"Dominican Republic"                           => "DO",
			"Ecuador"                                      => "EC",
			"Egypt"                                        => "EG",
			"El Salvador"                                  => "SV",
			"Equatorial Guinea"                            => "GQ",
			"Eritrea"                                      => "ER",
			"Estonia"                                      => "EE",
			"Ethiopia"                                     => "ET",
			"Falkland Islands (Malvinas)"                  => "FK",
			"Faroe Islands"                                => "FO",
			"Fiji"                                         => "FJ",
			"Finland"                                      => "FI",
			"France"                                       => "FR",
			"French Guiana"                                => "GF",
			"French Polynesia"                             => "PF",
			"French Southern Territories"                  => "TF",
			"Gabon"                                        => "GA",
			"Gambia"                                       => "GM",
			"Georgia"                                      => "GE",
			"Germany"                                      => "DE",
			"Ghana"                                        => "GH",
			"Gibraltar"                                    => "GI",
			"Greece"                                       => "GR",
			"Greenland"                                    => "GL",
			"Grenada"                                      => "GD",
			"Guadeloupe"                                   => "GP",
			"Guam"                                         => "GU",
			"Guatemala"                                    => "GT",
			"Guernsey"                                     => "GG",
			"Guinea"                                       => "GN",
			"Guinea-Bissau"                                => "GW",
			"Guyana"                                       => "GY",
			"Haiti"                                        => "HT",
			"Heard Island and Mcdonald Islands"            => "HM",
			"Vatican City"                                 => "VA",
			"Honduras"                                     => "HN",
			"Hong Kong"                                    => "HK",
			"Hungary"                                      => "HU",
			"Iceland"                                      => "IS",
			"India"                                        => "IN",
			"Indonesia"                                    => "ID",
			"Iran"                                         => "IR",
			"Iraq"                                         => "IQ",
			"Ireland"                                      => "IE",
			"Isle of Man"                                  => "IM",
			"Israel"                                       => "IL",
			"Italy"                                        => "IT",
			"Jamaica"                                      => "JM",
			"Japan"                                        => "JP",
			"Jersey"                                       => "JE",
			"Hashemite Kingdom of Jordan"                  => "JO",
			"Kazakhstan"                                   => "KZ",
			"Kenya"                                        => "KE",
			"Kiribati"                                     => "KI",
			"Korea, Democratic People'S Republic of"       => "KP",
			"Republic of Korea"                            => "KR",
			"South Korea"                                  => "KR",
			"Kuwait"                                       => "KW",
			"Kyrgyzstan"                                   => "KG",
			"Laos"                                         => "LA",
			"Latvia"                                       => "LV",
			"Lebanon"                                      => "LB",
			"Lesotho"                                      => "LS",
			"Liberia"                                      => "LR",
			"Libyan Arab Jamahiriya"                       => "LY",
			"Liechtenstein"                                => "LI",
			"Republic of Lithuania"                        => "LT",
			"Luxembourg"                                   => "LU",
			"Macao"                                        => "MO",
			"Macedonia, The Former Yugoslav Republic of"   => "MK",
			"Madagascar"                                   => "MG",
			"Malawi"                                       => "MW",
			"Malaysia"                                     => "MY",
			"Maldives"                                     => "MV",
			"Mali"                                         => "ML",
			"Malta"                                        => "MT",
			"Marshall Islands"                             => "MH",
			"Martinique"                                   => "MQ",
			"Mauritania"                                   => "MR",
			"Mauritius"                                    => "MU",
			"Mayotte"                                      => "YT",
			"Mexico"                                       => "MX",
			"Micronesia, Federated States of"              => "FM",
			"Republic of Moldova"                          => "MD",
			"Monaco"                                       => "MC",
			"Mongolia"                                     => "MN",
			"Montenegro"                                   => "ME",
			"Montserrat"                                   => "MS",
			"Morocco"                                      => "MA",
			"Mozambique"                                   => "MZ",
			"Myanmar"                                      => "MM",
			"Namibia"                                      => "NA",
			"Nauru"                                        => "NR",
			"Nepal"                                        => "NP",
			"Netherlands"                                  => "NL",
			"Netherlands Antilles"                         => "AN",
			"New Caledonia"                                => "NC",
			"New Zealand"                                  => "NZ",
			"Nicaragua"                                    => "NI",
			"Niger"                                        => "NE",
			"Nigeria"                                      => "NG",
			"Niue"                                         => "NU",
			"Norfolk Island"                               => "NF",
			"Northern Mariana Islands"                     => "MP",
			"Norway"                                       => "NO",
			"Oman"                                         => "OM",
			"Pakistan"                                     => "PK",
			"Palau"                                        => "PW",
			"Palestine"                                    => "PS",
			"Panama"                                       => "PA",
			"Papua New Guinea"                             => "PG",
			"Paraguay"                                     => "PY",
			"Peru"                                         => "PE",
			"Philippines"                                  => "PH",
			"Pitcairn"                                     => "PN",
			"Poland"                                       => "PL",
			"Portugal"                                     => "PT",
			"Puerto Rico"                                  => "PR",
			"Qatar"                                        => "QA",
			"Reunion"                                      => "RE",
			"Romania"                                      => "RO",
			"Russia"                                       => "RU",
			"RWANDA"                                       => "RW",
			"Saint Helena"                                 => "SH",
			"Saint Kitts and Nevis"                        => "KN",
			"Saint Lucia"                                  => "LC",
			"Saint Pierre and Miquelon"                    => "PM",
			"Saint Vincent and the Grenadines"             => "VC",
			"Samoa"                                        => "WS",
			"San Marino"                                   => "SM",
			"Sao Tome and Principe"                        => "ST",
			"Saudi Arabia"                                 => "SA",
			"Senegal"                                      => "SN",
			"Serbia"                                       => "RS",
			"Seychelles"                                   => "SC",
			"Sierra Leone"                                 => "SL",
			"Singapore"                                    => "SG",
			"Slovakia"                                     => "SK",
			"Slovenia"                                     => "SI",
			"Solomon Islands"                              => "SB",
			"Somalia"                                      => "SO",
			"South Africa"                                 => "ZA",
			"South Georgia and the South Sandwich Islands" => "GS",
			"Spain"                                        => "ES",
			"Sri Lanka"                                    => "LK",
			"Sudan"                                        => "SD",
			"Suriname"                                     => "SR",
			"Svalbard and Jan Mayen"                       => "SJ",
			"Swaziland"                                    => "SZ",
			"Sweden"                                       => "SE",
			"Switzerland"                                  => "CH",
			"Syrian Arab Republic"                         => "SY",
			"Taiwan"                                       => "TW",
			"Tajikistan"                                   => "TJ",
			"Tanzania"                                     => "TZ",
			"Thailand"                                     => "TH",
			"Timor-Leste"                                  => "TL",
			"Togo"                                         => "TG",
			"Tokelau"                                      => "TK",
			"Tonga"                                        => "TO",
			"Trinidad and Tobago"                          => "TT",
			"Tunisia"                                      => "TN",
			"Turkey"                                       => "TR",
			"Turkmenistan"                                 => "TM",
			"Turks and Caicos Islands"                     => "TC",
			"Tuvalu"                                       => "TV",
			"Uganda"                                       => "UG",
			"Ukraine"                                      => "UA",
			"United Arab Emirates"                         => "AE",
			"United Kingdom"                               => "GB",
			"United States"                                => "US",
			"United States Minor Outlying Islands"         => "UM",
			"Uruguay"                                      => "UY",
			"Uzbekistan"                                   => "UZ",
			"Vanuatu"                                      => "VU",
			"Venezuela"                                    => "VE",
			"Vietnam"                                      => "VN",
			"Virgin Islands, British"                      => "VG",
			"Virgin Islands, U.S."                         => "VI",
			"Wallis and Futuna"                            => "WF",
			"Western Sahara"                               => "EH",
			"Yemen"                                        => "YE",
			"Zambia"                                       => "ZM",
			"Zimbabwe"                                     => "ZW",
		];

		return wp_json_encode( $allCountries );

	}


	/**
	 * Advanced Page Visit Counter Get short country name from full country
	 * name to list on reports pages.
	 *
	 * @since    3.0.1
	 */
	public function get_country_name( $name ) {

		$allCountries = json_decode( $this->get_country_names(), TRUE );
		if ( ! empty( $name ) ) {
			return $allCountries[ $name ];
		} else {
			return '';
		}
	}

	/**
	 * Advanced Page Visit Counter Get Registered Post Types to list on reports
	 * pages.
	 *
	 * @since    3.0.1
	 */
	public function apvc_get_post_types__premium_only() {
		global $wpdb;
		$posttypes = get_post_types( [ 'public' => TRUE ] );

		return wp_json_encode( $posttypes );
	}

	/**
	 * Advanced Page Visit Counter Get Posts from selected post types to list
	 * on reports pages.
	 *
	 * @since    3.0.1
	 */
	public function apvc_get_posts__premium_only() {
		global $wpdb;

		if ( ! empty( $_GET['pts'] ) ) {
			$psts = explode( ",", ( sanitize_text_field( $_GET['pts'] ) ) );
		} else {
			$psts = "post";
		}

		$allPosts = get_posts( [
			"post_type"      => $psts,
			"post_status"    => "publish",
			"posts_per_page" => "-1",
			"fields"         => [ "titles", "ids" ],
		] );

		return wp_json_encode( $allPosts );
	}

	/**
	 * Advanced Page Visit Counter Get total counts of the last month.
	 *
	 * @since    3.0.1
	 */
	public function get_total_counts_of_last_month() {
		global $wpdb;
		$table             = APVC_DATA_TABLE;
		$currentDate       = date( 'Y-m-d H:i:s', strtotime( date( 'Y-m-d 0:0:0' ) . ' -1 month' ) );
		$prevMonthDate     = date( 'Y-m-d H:i:s', strtotime( date( 'Y-m-d 0:0:0' ) . ' -2 month' ) );
		$prevPrevMonthDate = date( 'Y-m-d H:i:s', strtotime( date( 'Y-m-d 0:0:0' ) . ' -3 month' ) );

		$lastMonthCounts     = $wpdb->get_var( "SELECT COUNT(*) as total_count FROM {$table} WHERE date >= '" . $currentDate . "'" );
		$prevMonthCounts     = $wpdb->get_var( "SELECT COUNT(*) as total_count FROM {$table} WHERE date >= '" . $prevMonthDate . "' AND date <= '" . $currentDate . "'" );
		$prevPrevMonthCounts = $wpdb->get_var( "SELECT COUNT(*) as total_count FROM {$table} WHERE date >= '" . $prevPrevMonthDate . "' AND date <= '" . $prevMonthDate . "'" );

		$diff  = $lastMonthCounts - $prevMonthCounts;
		$diff  = ( $diff >= 0 ) ? "+" . $diff : $diff;
		$class = ( $diff >= 0 ) ? "text-success" : "text-danger";

		return wp_json_encode( [
			"lastMonth"   => $lastMonthCounts,
			"months_wise" => [
				$prevPrevMonthCounts,
				$prevMonthCounts,
				$lastMonthCounts,
			],
			"countDiff"   => $diff,
			"class"       => $class,
		] );
	}

	/**
	 * Advanced Page Visit Counter Get total counts of the last week.
	 *
	 * @since    3.0.1
	 */
	public function get_total_counts_of_last_week() {
		global $wpdb;
		$table = APVC_DATA_TABLE;

		$currentDate      = date( 'Y-m-d H:i:s', strtotime( date( 'Y-m-d 0:0:0' ) . ' -1 week' ) );
		$prevWeekDate     = date( 'Y-m-d H:i:s', strtotime( date( 'Y-m-d 0:0:0' ) . ' -2 weeks' ) );
		$prevPrevWeekDate = date( 'Y-m-d H:i:s', strtotime( date( 'Y-m-d 0:0:0' ) . ' -3 weeks' ) );

		$lastWeekCounts     = $wpdb->get_var( "SELECT COUNT(*) as total_count FROM {$table} WHERE date >= '" . $currentDate . "'" );
		$prevWeekCounts     = $wpdb->get_var( "SELECT COUNT(*) as total_count FROM {$table} WHERE date >= '" . $prevWeekDate . "' AND date <= '" . $currentDate . "'" );
		$prevPrevWeekCounts = $wpdb->get_var( "SELECT COUNT(*) as total_count FROM {$table} WHERE date >= '" . $prevPrevWeekDate . "' AND date <= '" . $prevWeekDate . "'" );

		$diff  = $lastWeekCounts - $prevWeekCounts;
		$diff  = ( $diff >= 0 ) ? "+" . $diff : $diff;
		$class = ( $diff >= 0 ) ? "text-success" : "text-danger";

		return wp_json_encode( [
			"lastWeek"   => $lastWeekCounts,
			"weeks_wise" => [
				$prevPrevWeekCounts,
				$prevWeekCounts,
				$lastWeekCounts,
			],
			"countDiff"  => $diff,
			"class"      => $class,
		] );
	}

	/**
	 * Advanced Page Visit Counter Get total counts of daily.
	 *
	 * @since    3.0.1
	 */
	public function get_total_counts_of_last_daily() {
		global $wpdb;
		$table = APVC_DATA_TABLE;


		$currentDate   = date( 'Y-m-d H:i:s', strtotime( date( 'Y-m-d 0:0:0' ) ) );
		$yesterdayDate = date( 'Y-m-d H:i:s', strtotime( date( 'Y-m-d 0:0:0' ) . ' -2 day' ) );
		$prevDate      = date( 'Y-m-d H:i:s', strtotime( date( 'Y-m-d 0:0:0' ) . ' -3 day' ) );

		$todaysCounts    = $wpdb->get_var( "SELECT COUNT(*) as total_count FROM {$table} WHERE date >= '" . $currentDate . "'" );
		$yesterdayCounts = $wpdb->get_var( "SELECT COUNT(*) as total_count FROM {$table} WHERE date >= '" . $yesterdayDate . "' AND date <= '" . $currentDate . "'" );
		$prevDate        = $wpdb->get_var( "SELECT COUNT(*) as total_count FROM {$table} WHERE date >= '" . $prevDate . "' AND date <= '" . $yesterdayDate . "'" );

		$diff  = $todaysCounts - $yesterdayCounts;
		$diff  = ( $diff >= 0 ) ? "+" . $diff : $diff;
		$class = ( $diff >= 0 ) ? "text-success" : "text-danger";

		return wp_json_encode( [
			"todaysCounts" => $todaysCounts,
			"day_wise"     => [
				$prevDate,
				$yesterdayCounts,
				$todaysCounts,
			],
			"countDiff"    => $diff,
			"class"        => $class,
		] );

	}

	/**
	 * Advanced Page Visit Counter Show RealTime visitors data.
	 *
	 * @since    3.0.1
	 */
	public function apvc_refresh_live_visitors__premium_only() {

		if ( ! wp_verify_nonce( $_POST['security'], 'security_nonce' ) ) {
			die( 'Permission Denied.' );
		}

		global $wpdb;
		$apvc_table      = APVC_DATA_TABLE;
		$table           = APVC_REALTIME_USERS_TABLE;
		$currentDate     = date( 'H:i:s', strtotime( "-5 minute" ) );
		$currentDateFull = date( 'Y-m-d H:i:s', strtotime( "-10 minute" ) );

		$wpdb->query( "DELETE FROM {$table} WHERE date <= '$currentDateFull'" );
		$realtimeUsers = count( $wpdb->get_results( "SELECT ip_address FROM {$table} WHERE time >= '$currentDate' GROUP BY ip_address" ) );

		$realtimeUsersLog = $wpdb->get_results( "SELECT (SELECT post_title FROM $wpdb->posts WHERE ID=ref_id) AS title, ref_id FROM {$table} WHERE time >= '$currentDate' GROUP BY ref_id ORDER BY date DESC LIMIT 0,10" );

		$rtCountries = $wpdb->get_results( "SELECT country FROM {$table} WHERE time >= '$currentDate' AND country != '' AND ip_address != '' GROUP BY country ORDER BY date DESC LIMIT 0,10" );

		$allCountries = [];
		foreach ( $rtCountries as $cnt ):

			$flagT = APVC_PLUGIN_PATH . "admin/images/flags/" . strtolower( $this->get_country_name( $cnt->country ) ) . ".svg";

			if ( ! file_exists( $flagT ) ) {
				$flag = APVC_PLUGIN_URL . "admin/images/flags/default-flag.png";
			} else {
				$flag = APVC_PLUGIN_URL . "admin/images/flags/" . strtolower( $this->get_country_name( $cnt->country ) ) . ".svg";
			}
			array_push( $allCountries, [
					"name" => $cnt->country,
					"flag" => $flag,
				]
			);
		endforeach;

		$realtimeUsers = ( $realtimeUsers > 0 ) ? $realtimeUsers : 0;

		echo wp_json_encode( [
			"liveUsers"   => $realtimeUsers,
			"UserLog"     => $realtimeUsersLog,
			"rtCountries" => $allCountries,
		] );

		wp_die();
	}

	/**
	 * Advanced Page Visit Counter Get total counts.
	 *
	 * @since    3.0.1
	 */
	public function get_total_counts() {
		global $wpdb;

		$table       = APVC_DATA_TABLE;
		$total_count = $wpdb->get_var( "SELECT COUNT(*) FROM {$table}" );

		return $total_count;
		wp_die();
	}

	/**
	 * Advanced Page Visit Counter Get browser traffic stats.
	 *
	 * @since    3.0.1
	 */
	public function get_browser_traffic_stats() {
		global $wpdb;
		$table = APVC_DATA_TABLE;

		$browserStats = $wpdb->get_results( "SELECT COUNT(*) as total_count, device_type, COUNT(*)*100/(SELECT COUNT(*) FROM {$table}) as percentage  FROM {$table} GROUP BY device_type" );

		return wp_json_encode( $browserStats );
	}

	/**
	 * Advanced Page Visit Counter Get browser traffic stats list.
	 *
	 * @since    3.0.1
	 */
	public function get_browser_traffic_stats_list() {
		global $wpdb;
		$table = APVC_DATA_TABLE;

		$browserStatsList = $wpdb->get_results( "SELECT COUNT(*) as total_count, browser_short_name, browser_full_name FROM {$table} GROUP BY browser_short_name order by total_count DESC LIMIT 0,5" );

		return wp_json_encode( $browserStatsList );
	}

	/**
	 * Advanced Page Visit Counter Get referrel websites traffic stats.
	 *
	 * @since    3.0.1
	 */
	public function get_referral_websites_stats() {
		global $wpdb;
		$table = APVC_DATA_TABLE;

		$referralStats = $wpdb->get_results( "SELECT COUNT(*) as total_count, SUBSTRING_INDEX(SUBSTRING_INDEX(REPLACE(REPLACE(LOWER(http_referer), 'https://', ''), 'http://', ''), '/', 1), '?', 1) as http_referer, http_referer as htp_ref, COUNT(*)*100/(SELECT COUNT(*) FROM {$table}) as percentage FROM {$table} GROUP BY http_referer ORDER BY total_count DESC LIMIT 0,5 " );

		return wp_json_encode( $referralStats );
	}

	/**
	 * Advanced Page Visit Counter Get stats by Operation Systems.
	 *
	 * @since    3.0.1
	 */
	public function get_stats_by_operating_systems() {
		global $wpdb;
		$table = APVC_DATA_TABLE;

		$OsStats = $wpdb->get_results( "SELECT COUNT(*) as total_count, operating_system, COUNT(*)*100/(SELECT COUNT(*) FROM {$table}) as percentage FROM {$table} GROUP BY operating_system ORDER BY total_count DESC LIMIT 0,5" );

		return wp_json_encode( $OsStats );
	}

	/**
	 * Advanced Page Visit Counter Get recent visit stats.
	 *
	 * @since    3.0.1
	 */
	public function get_recent_visit( $article_id, $type = '' ) {
		global $wpdb;
		$table = APVC_DATA_TABLE;

		if ( $type == 'ip_address' ) {
			$recentVisit = $wpdb->get_var( "SELECT date FROM {$table} WHERE ip_address = '$article_id' ORDER BY date DESC" );
		} else {
			$recentVisit = $wpdb->get_var( "SELECT date FROM {$table} WHERE article_id = $article_id ORDER BY date DESC" );
		}

		$recentVisit = human_time_diff( strtotime( $recentVisit ), strtotime( date( "Y-m-d 0:0:0" ) ) ) . __( ' ago', "advanced-page-visit-counter" );

		return $recentVisit;
	}

	/**
	 * Advanced Page Visit Counter Get total reports of all countries.
	 *
	 * @since    3.0.1
	 */
	public function get_the_reports_for_countries__premium_only( $offset, $no_of_records_per_page ) {
		global $wpdb;
		$table = APVC_DATA_TABLE;

		$where = '';

		$d = ( isset( $_REQUEST['d'] ) ) ? sanitize_text_field( $_REQUEST['d'] ) : '';
		$e = ( isset( $_REQUEST['e'] ) ) ? sanitize_text_field( $_REQUEST['e'] ) : '';

		if ( $d != 0 && ! $this->apvc_is_date( $d ) ) {
			$days_ago = date( 'Y-m-d', time() - (int)$d * (int)APVC_SECONDS_PER_DAY );
			$where    = " WHERE date >= '$days_ago' ";
		} else if ( $this->apvc_is_date( $d ) && $this->apvc_is_date( $e ) ) {
			$where = " WHERE date >= '$d' AND date <= '$e' ";
		} else {
			$where = " ";
		}

		$totalCount = count( $wpdb->get_results( "SELECT article_id FROM {$table} " . $where . " GROUP BY country" ) );

		$apvcReports = $wpdb->get_results( "SELECT COUNT(*) as count, country FROM {$table} " . $where . " GROUP BY country ORDER BY count DESC LIMIT $offset, $no_of_records_per_page" );

		return wp_json_encode( [
			"totalCount" => $totalCount,
			"list"       => $apvcReports,
		] );

	}


	/**
	 * Advanced Page Visit Counter Get total reports.
	 *
	 * @since    3.0.1
	 */
	public function get_the_reports( $offset, $no_of_records_per_page ) {
		global $wpdb;
		$table = APVC_DATA_TABLE;

		$where = '';
		if ( apvc_fs()->is__premium_only() ) {

			$d = ( isset( $_REQUEST['d'] ) ) ? sanitize_text_field( $_REQUEST['d'] ) : '';
			$e = ( isset( $_REQUEST['e'] ) ) ? sanitize_text_field( $_REQUEST['e'] ) : '';

			if ( $d != 0 && ! $this->apvc_is_date( $d ) ) {
				$days_ago = date( 'Y-m-d', time() - intval($d) * APVC_SECONDS_PER_DAY );
				$where    = " WHERE date >= '$days_ago' ";
			} else if ( $this->apvc_is_date( $d ) && $this->apvc_is_date( $e ) ) {
				$where = " WHERE date >= '$d' AND date <= '$e' ";
			} else {
				$where = " ";
			}
		} else {
			$where = " ";
		}

		$totalCount = count( $wpdb->get_results( "SELECT article_id FROM {$table} " . $where . " GROUP BY article_id" ) );

		$apvcReports = $wpdb->get_results( "SELECT COUNT(*) as count, article_id, ( SELECT post_title FROM {$wpdb->prefix}posts WHERE ID=article_id ) as title FROM {$table} " . $where . " GROUP BY article_id ORDER BY count DESC LIMIT $offset, $no_of_records_per_page" );

		return wp_json_encode( [
			"totalCount" => $totalCount,
			"list"       => $apvcReports,
		] );

	}

	/**
	 * Advanced Page Visit Counter Get detailed report of any article.
	 *
	 * @since    3.0.1
	 */
	public function get_the_detailed_reports( $article_id, $offset, $no_of_records_per_page ) {
		global $wpdb;
		$table = APVC_DATA_TABLE;

		$where = '';
		if ( apvc_fs()->is__premium_only() ) {

			$d = ( isset( $_REQUEST['d'] ) ) ? sanitize_text_field( $_REQUEST['d'] ) : '';
			$e = ( isset( $_REQUEST['e'] ) ) ? sanitize_text_field( $_REQUEST['e'] ) : '';

			$u_g = ( isset( $_REQUEST['u_g'] ) ) ? sanitize_text_field( $_REQUEST['u_g'] ) : '';
			$u_r = ( isset( $_REQUEST['u_r'] ) ) ? sanitize_text_field( $_REQUEST['u_r'] ) : '';

			$cnt      = ( isset( $_REQUEST['cnt'] ) && $_REQUEST['cnt'] != '' ) ? sanitize_text_field( implode( "','", explode( ",", $_REQUEST['cnt'] ) ) ) : "";
			$dv       = ( isset( $_REQUEST['dv'] ) && $_REQUEST['dv'] != '' ) ? sanitize_text_field( implode( "','", explode( ",", $_REQUEST['dv'] ) ) ) : "";
			$br       = ( isset( $_REQUEST['br'] ) && $_REQUEST['br'] != '' ) ? sanitize_text_field( implode( "','", explode( ",", $_REQUEST['br'] ) ) ) : "";
			$os       = ( isset( $_REQUEST['os'] ) && $_REQUEST['os'] != '' ) ? sanitize_text_field( implode( "','", explode( ",", $_REQUEST['os'] ) ) ) : "";
			$subQuery = "";
			$country  = ( isset( $_GET['country'] ) ) ? sanitize_text_field( $_GET['country'] ) : "";

			if ( $d != 0 && ! $this->apvc_is_date( $d ) ) {
				$days_ago = date( 'Y-m-d', time() - (int)$d * (int)APVC_SECONDS_PER_DAY );
				$where    .= " AND date >= '$days_ago' ";
			}
			if ( $this->apvc_is_date( $d ) && $this->apvc_is_date( $e ) ) {
				$where .= " AND date >= '$d' AND date <= '$e' ";
			}

			if ( $cnt != '' ) {
				$where .= " AND country IN ('$cnt') ";
			}

			if ( $dv != '' ) {
				$where .= " AND device_type IN ('$dv') ";
			}

			if ( $br != '' ) {
				$where .= " AND browser_short_name IN ('$br') ";
			}

			if ( $os != '' ) {
				$where .= " AND operating_system IN ('$os') ";
			}

			if ( $u_g == 'on' ) {
				$where .= " AND user_type = 'Guest' ";
			}

			if ( $u_r == 'on' ) {
				$where .= " AND user_type = 'Registered' ";
			}

			if ( $country !== '' ) {
				$where .= " AND country = '$country' ";
			}

		} else {
			$where = " ";
		}

		$totalCount = count( $wpdb->get_results( "SELECT article_id FROM {$table} WHERE article_id=$article_id " . $where . " " ) );

		$apvcReports = $wpdb->get_results( "SELECT *, (SELECT post_title FROM {$wpdb->prefix}posts WHERE ID=$article_id) as title,  SUBSTRING_INDEX(SUBSTRING_INDEX(REPLACE(REPLACE(LOWER(http_referer), 'https://', ''), 'http://', ''), '/', 1), '?', 1) as http_referer_clean FROM {$table} WHERE article_id=$article_id " . $where . " ORDER BY date DESC LIMIT $offset, $no_of_records_per_page" );

		return wp_json_encode( [
			"totalCount" => $totalCount,
			"list"       => $apvcReports,
		] );
	}


	/**
	 * Advanced Page Visit Counter Get total sales for WooCommerce Widget.
	 *
	 * @since    3.0.1
	 */
	public function apvc_get_wc_total_sales_total__premium_only() {
		global $wpdb;

		$order_totals = get_transient( "apvc_orders_total" );
		if ( empty( $order_totals ) ) {

			$order_totals = apply_filters( 'woocommerce_reports_sales_overview_order_totals', $wpdb->get_row( "

			SELECT SUM(meta.meta_value) AS total_sales, COUNT(posts.ID) AS total_orders FROM {$wpdb->posts} AS posts

			LEFT JOIN {$wpdb->postmeta} AS meta ON posts.ID = meta.post_id

			WHERE meta.meta_key = '_order_total'

			AND posts.post_type = 'shop_order'

			AND posts.post_status IN ( '" . implode( "','", [
					'wc-completed',
					'wc-processing',
					'wc-on-hold',
				] ) . "' )

			" ) );
			set_transient( "apvc_orders_total", absint( $order_totals->total_sales ), APVC_HOURLY_REFRESH );
			$order_totals = absint( $order_totals->total_sales );
		}

		return $order_totals;
	}

	/**
	 * Advanced Page Visit Counter Get total orders for WooCommerce Widget.
	 *
	 * @since    3.0.1
	 */
	public function apvc_get_wc_total_orders_count__premium_only() {
		global $wpdb;

		$order_totals = get_transient( "apvc_total_orders_data" );
		if ( empty( $order_totals ) ) {

			$order_totals = apply_filters( 'woocommerce_reports_sales_overview_order_totals', $wpdb->get_row( "
		    SELECT COUNT(posts.ID) AS total_orders FROM {$wpdb->posts} AS posts

		    LEFT JOIN {$wpdb->postmeta} AS meta ON posts.ID = meta.post_id
		    LEFT JOIN {$wpdb->term_relationships} AS rel ON posts.ID=rel.object_ID
		    LEFT JOIN {$wpdb->term_taxonomy} AS tax USING( term_taxonomy_id )
		    LEFT JOIN {$wpdb->terms} AS term USING( term_id )

		    WHERE   meta.meta_key       = '_order_total'
		    AND     posts.post_type     = 'shop_order'
		    AND     posts.post_status   = 'publish'
		    AND     tax.taxonomy        = 'shop_order_status'
		    AND     term.slug           IN ('" . implode( "','",
					apply_filters( 'woocommerce_reports_order_statuses',
						[ 'completed', 'processing', 'on-hold' ] ) ) . "')
			" ) );

			set_transient( "apvc_total_orders_data", absint( $order_totals->total_orders ), APVC_HOURLY_REFRESH );
			$order_totals = absint( $order_totals->total_orders );
		}

	}

	/**
	 * Advanced Page Visit Counter Get total products sales for WooCommerce
	 * Widget.
	 *
	 * @since    3.0.1
	 */
	public function apvc_get_total_products_sales__premium_only() {
		global $wpdb;
		global $woocommerce;
		$totalProducts = get_transient( "apvc_total_products_sales" );
		if ( empty( $totalProducts ) || 0 ) {
			$orders        = get_posts( [
				'numberposts' => - 1,
				'post_type'   => wc_get_order_types(),
				'post_status' => array_keys( wc_get_order_statuses() ),
				'fields'      => 'ids',
			] );
			$totalProducts = 0;
			foreach ( $orders as $orderID ):
				$order = wc_get_order( $orderID );
				foreach ( $order->get_items() as $item_id => $item_data ) {
					$totalProducts += $item_data->get_quantity();
				}
			endforeach;
			set_transient( "apvc_total_products_sales", $totalProducts, APVC_HOURLY_REFRESH );
			$totalProducts = $totalProducts;
		}
		echo $totalProducts;
	}

	/**
	 * Advanced Page Visit Counter Get top 10 pages data.
	 *
	 * @since    3.0.1
	 */
	public function apvc_get_top_10_page_data() {
		global $wpdb;
		$table = APVC_DATA_TABLE;

		$where = '';
		if ( apvc_fs()->is__premium_only() ) {

			$d = (isset($_REQUEST['d'])) ? sanitize_text_field( $_REQUEST['d'] ) : "";
			$e = (isset($_REQUEST['e'])) ? sanitize_text_field( $_REQUEST['e'] ) : "";

			if ( $d != 0 && ! $this->apvc_is_date( $d ) ) {
				$days_ago = date( 'Y-m-d', time() - (int)$d * (int)APVC_SECONDS_PER_DAY );
				$where    = " WHERE article_id != '' AND article_type = 'page' AND date >= '$days_ago' ";
			} else if ( $this->apvc_is_date( $d ) && $this->apvc_is_date( $e ) ) {
				$where = " WHERE article_id != '' AND article_type = 'page' AND date >= '$d' AND date <= '$e' ";
			} else {
				$where = "  WHERE article_id != '' AND article_type = 'page' ";
			}
		} else {
			$where = "  WHERE article_id != '' AND article_type = 'page' ";
		}

		$top10Pages = $wpdb->get_results( "SELECT article_id, count(*) as count, ( SELECT post_title FROM {$wpdb->prefix}posts WHERE ID=article_id ) as title FROM {$table} " . $where . " GROUP BY article_id ORDER BY count DESC LIMIT 10" );

		return wp_json_encode( $top10Pages );
	}

	/**
	 * Advanced Page Visit Counter Get top 10 posts data.
	 *
	 * @since    3.0.1
	 */
	public function apvc_get_top_10_posts_data() {
		global $wpdb;
		$table = APVC_DATA_TABLE;

		$where = '';
		if ( apvc_fs()->is__premium_only() ) {

			$d = (isset($_REQUEST['d'])) ? sanitize_text_field( $_REQUEST['d'] ) : "";
			$e = (isset($_REQUEST['e'])) ? sanitize_text_field( $_REQUEST['e'] ) : "";

			if ( isset( $_REQUEST['pts'] ) && ! empty( $_REQUEST['pts'] ) ) {
				$ptsAr = explode( ",", sanitize_text_field( $_REQUEST['pts'] ) );
				if ( ( $key = array_search( 'page', $ptsAr ) ) !== FALSE ) {
					unset( $ptsAr[ $key ] );
				}
				$p   = implode( "','", $ptsAr );
				$pQr = " AND article_type IN ('$p') ";
			} else {
				$pQr = "";
			}

			if ( $d != 0 && ! $this->apvc_is_date( $d ) ) {
				$days_ago = date( 'Y-m-d', time() - (int)$d * (int)APVC_SECONDS_PER_DAY );
				$where    = " WHERE article_id != '' AND article_type NOT IN ('page','cart') AND date >= '$days_ago' $pQr";
			} else if ( $this->apvc_is_date( $d ) && $this->apvc_is_date( $e ) ) {
				$where = " WHERE article_id != '' AND article_type NOT IN ('page','cart') AND date >= '$d' AND date <= '$e' $pQr";
			} else {
				$where = "  WHERE article_id != '' AND article_type NOT IN ('page','cart') $pQr";
			}
		} else {
			$where = " WHERE article_id != '' AND article_type NOT IN ('page','cart') ";
		}


		$top10Posts = $wpdb->get_results( "SELECT article_id, count(*) as count, ( SELECT post_title FROM {$wpdb->prefix}posts WHERE ID=article_id ) as title FROM {$table} " . $where . " GROUP BY article_id ORDER BY count DESC LIMIT 10" );

		return wp_json_encode( $top10Posts );
	}

	/**
	 * Advanced Page Visit Counter Get top 10 countries data.
	 *
	 * @since    3.0.1
	 */
	public function apvc_get_top_10_contries_data() {
		global $wpdb;
		$table = APVC_DATA_TABLE;

		$where = '';
		if ( apvc_fs()->is__premium_only() ) {

			$d = (isset($_REQUEST['d'])) ? sanitize_text_field( $_REQUEST['d'] ) : "";
			$e = (isset($_REQUEST['e'])) ? sanitize_text_field( $_REQUEST['e'] ) : "";

			if ( isset( $_REQUEST['pts'] ) && ! empty( $_REQUEST['pts'] ) ) {
				$ptsAr = explode( ",", sanitize_text_field( $_REQUEST['pts'] ) );
				$p     = implode( "','", $ptsAr );
				$pQr   = " AND article_type IN ('$p') ";
			} else {
				$pQr = "";
			}

			if ( isset( $_REQUEST['psts'] ) && ! empty( $_REQUEST['psts'] ) ) {
				$pstsAr = explode( ",", sanitize_text_field( $_REQUEST['psts'] ) );
				$ps     = implode( "','", $pstsAr );
				$pSts   = " AND article_id IN ('$ps') ";
			} else {
				$pSts = "";
			}

			if ( $d != 0 && ! $this->apvc_is_date( $d ) ) {
				$days_ago = date( 'Y-m-d', time() - (int)$d * (int)APVC_SECONDS_PER_DAY );
				$where    = " WHERE country != '' AND date >= '$days_ago' $pQr $pSts ";
			} else if ( $this->apvc_is_date( $d ) && $this->apvc_is_date( $e ) ) {
				$where = " WHERE country != '' AND date >= '$d' AND date <= '$e' $pQr $pSts ";
			} else {
				$where = "  WHERE country != '' $pQr $pSts ";
			}
		} else {
			$where = " WHERE country != '' ";
		}

		$top10Country = $wpdb->get_results( "SELECT article_id, country, count(*) as count FROM {$table} " . $where . "  GROUP BY country ORDER BY count DESC LIMIT 10" );

		return wp_json_encode( $top10Country );
	}

	/**
	 * Advanced Page Visit Counter Get top 10 ip address data.
	 *
	 * @since    3.0.1
	 */
	public function apvc_get_top_10_ip_address_data() {
		global $wpdb;
		$table = APVC_DATA_TABLE;

		$where = '';
		if ( apvc_fs()->is__premium_only() ) {

			$d = (isset($_REQUEST['d'])) ? sanitize_text_field( $_REQUEST['d'] ) : "";
			$e = (isset($_REQUEST['e'])) ? sanitize_text_field( $_REQUEST['e'] ) : "";

			if ( isset( $_REQUEST['pts'] ) && ! empty( $_REQUEST['pts'] ) ) {
				$ptsAr = explode( ",", sanitize_text_field( $_REQUEST['pts'] ) );
				$p     = implode( "','", $ptsAr );
				$pQr   = " AND article_type IN ('$p') ";
			} else {
				$pQr = "";
			}

			if ( isset( $_REQUEST['psts'] ) && ! empty( $_REQUEST['psts'] ) ) {
				$pstsAr = explode( ",", sanitize_text_field( $_REQUEST['psts'] ) );
				$ps     = implode( "','", $pstsAr );
				$pSts   = " AND article_id IN ('$ps') ";
			} else {
				$pSts = "";
			}

			if ( $d != 0 && ! $this->apvc_is_date( $d ) ) {
				$days_ago = date( 'Y-m-d', time() - (int)$d * (int)APVC_SECONDS_PER_DAY );
				$where    = " WHERE ip_address != '' AND date >= '$days_ago' $pQr $pSts ";
			} else if ( $this->apvc_is_date( $d ) && $this->apvc_is_date( $e ) ) {
				$where = " WHERE ip_address != '' AND date >= '$d' AND date <= '$e' $pQr $pSts ";
			} else {
				$where = "  WHERE ip_address != '' $pQr $pSts ";
			}
		} else {
			$where = " WHERE ip_address != '' ";
		}

		$top10iPAddress = $wpdb->get_results( "SELECT article_id, ip_address, country, count(*) as count FROM {$table} " . $where . " GROUP BY ip_address ORDER BY count DESC LIMIT 10" );

		return wp_json_encode( $top10iPAddress );
	}

	/**
	 * Advanced Page Visit Counter Get human time difference data.
	 *
	 * @since    3.0.1
	 */
	public function apvc_get_human_time_diff( $recentVisit ) {
		$recentVisit = human_time_diff( strtotime( $recentVisit ), strtotime( date( "Y-m-d H:i:s" ) ) ) . __( ' ago', "advanced-page-visit-counter" );

		return $recentVisit;
	}


	/**
	 * Advanced Page Visit Counter Get countries.
	 *
	 * @since    3.0.6
	 */
	public function get_countries__premium_only() {
		global $wpdb;
		$table = APVC_DATA_TABLE;

		$AllCountries = $wpdb->get_results( "SELECT country FROM {$table} GROUP BY country" );

		return wp_json_encode( $AllCountries );
	}

	/**
	 * Advanced Page Visit Counter Get devices.
	 *
	 * @since    3.0.6
	 */
	public function get_devices__premium_only() {
		global $wpdb;
		$table = APVC_DATA_TABLE;

		$allDevices = $wpdb->get_results( "SELECT device_type FROM {$table} GROUP BY device_type ORDER BY device_type ASC" );

		return wp_json_encode( $allDevices );
	}

	/**
	 * Advanced Page Visit Counter Get browsers.
	 *
	 * @since    3.0.6
	 */
	public function get_browsers__premium_only() {
		global $wpdb;
		$table = APVC_DATA_TABLE;

		$allBrowserNames = $wpdb->get_results( "SELECT browser_short_name FROM {$table} GROUP BY browser_short_name ORDER BY browser_short_name ASC" );

		return wp_json_encode( $allBrowserNames );
	}

	/**
	 * Advanced Page Visit Counter Get Operating Systems.
	 *
	 * @since    3.0.6
	 */
	public function get_os__premium_only() {
		global $wpdb;
		$table = APVC_DATA_TABLE;

		$allOs = $wpdb->get_results( "SELECT operating_system FROM {$table} GROUP BY operating_system ORDER BY operating_system ASC" );

		return wp_json_encode( $allOs );
	}

	/**
	 * Advanced Page Visit Counter Get recent visit from a specific country.
	 *
	 * @since    3.0.6
	 */
	public function get_recent_visit_from_country__premium_only( $country ) {

		global $wpdb;
		$table = APVC_DATA_TABLE;

		$recentVisit = $wpdb->get_var( "SELECT date FROM {$table} WHERE country = '$country' ORDER BY date DESC LIMIT 1" );

		$recentVisit = human_time_diff( strtotime( $recentVisit ), strtotime( date( "Y-m-d 0:0:0" ) ) ) . __( ' ago', "advanced-page-visit-counter" );

		return $recentVisit;

	}

	/**
	 * Advanced Page Visit Counter Get detailed report of any article.
	 *
	 * @since    3.0.6
	 */
	public function get_the_country_detailed_reports__premium_only( $country, $offset, $no_of_records_per_page ) {
		global $wpdb;
		$table = APVC_DATA_TABLE;

		$where = '';

		$d = (isset($_REQUEST['d'])) ? sanitize_text_field( $_REQUEST['d'] ) : "";
		$e = (isset($_REQUEST['e'])) ? sanitize_text_field( $_REQUEST['e'] ) : "";

		$u_g = sanitize_text_field( $_REQUEST['u_g'] );
		$u_r = sanitize_text_field( $_REQUEST['u_r'] );

		$cnt      = ( $_REQUEST['cnt'] != '' ) ? sanitize_text_field( implode( "','", explode( ",", $_REQUEST['cnt'] ) ) ) : "";
		$dv       = ( $_REQUEST['dv'] != '' ) ? sanitize_text_field( implode( "','", explode( ",", $_REQUEST['dv'] ) ) ) : "";
		$br       = ( $_REQUEST['br'] != '' ) ? sanitize_text_field( implode( "','", explode( ",", $_REQUEST['br'] ) ) ) : "";
		$os       = ( $_REQUEST['os'] != '' ) ? sanitize_text_field( implode( "','", explode( ",", $_REQUEST['os'] ) ) ) : "";
		$subQuery = "";

		if ( $d != 0 && ! $this->apvc_is_date( $d ) ) {
			$days_ago = date( 'Y-m-d', time() - (int)$d * (int)APVC_SECONDS_PER_DAY );
			$where    .= " AND date >= '$days_ago' ";
		}
		if ( $this->apvc_is_date( $d ) && $this->apvc_is_date( $e ) ) {
			$where .= " AND date >= '$d' AND date <= '$e' ";
		}

		if ( $cnt != '' ) {
			$where .= " AND country IN ('$cnt') ";
		}

		if ( $dv != '' ) {
			$where .= " AND device_type IN ('$dv') ";
		}

		if ( $br != '' ) {
			$where .= " AND browser_short_name IN ('$br') ";
		}

		if ( $os != '' ) {
			$where .= " AND operating_system IN ('$os') ";
		}

		if ( $u_g == 'on' ) {
			$where .= " AND user_type = 'Guest' ";
		}

		if ( $u_r == 'on' ) {
			$where .= " AND user_type = 'Registered' ";
		}

		$totalCount = $wpdb->get_var( "SELECT COUNT(*) FROM {$table} WHERE country='$country' " . $where . " " );

		$apvcReports = $wpdb->get_results( "SELECT *, SUBSTRING_INDEX(SUBSTRING_INDEX(REPLACE(REPLACE(LOWER(http_referer), 'https://', ''), 'http://', ''), '/', 1), '?', 1) as http_referer_clean FROM {$table} WHERE country='$country' " . $where . " ORDER BY date DESC LIMIT $offset, $no_of_records_per_page" );

		return wp_json_encode( [
			"totalCount" => $totalCount,
			"list"       => $apvcReports,
		] );
	}


	/**
	 * Advanced Page Visit Counter Get total reports.
	 *
	 * @since    3.0.1
	 */
	public function get_the_reports_for_countries( $offset, $no_of_records_per_page, $country ) {
		global $wpdb;
		$table = APVC_DATA_TABLE;

		$where = '';

		$d = (isset($_REQUEST['d'])) ? sanitize_text_field( $_REQUEST['d'] ) : "";
		$e = (isset($_REQUEST['e'])) ? sanitize_text_field( $_REQUEST['e'] ) : "";

		if ( $d != 0 && ! $this->apvc_is_date( $d ) ) {
			$days_ago = date( 'Y-m-d', time() - (int)$d * (int)APVC_SECONDS_PER_DAY );
			$where    = " WHERE date >= '$days_ago' AND country = '$country' ";
		} else if ( $this->apvc_is_date( $d ) && $this->apvc_is_date( $e ) ) {
			$where = " WHERE date >= '$d' AND date <= '$e' AND country = '$country' ";
		} else {
			$where = " WHERE country = '$country' ";
		}

		$totalCount = count( $wpdb->get_results( "SELECT article_id FROM {$table} " . $where . " GROUP BY article_id" ) );

		$apvcReports = $wpdb->get_results( "SELECT COUNT(*) as count, article_id, ( SELECT post_title FROM {$wpdb->prefix}posts WHERE ID=article_id ) as title FROM {$table} " . $where . " GROUP BY article_id ORDER BY count DESC LIMIT $offset, $no_of_records_per_page" );

		return wp_json_encode( [
			"totalCount" => $totalCount,
			"list"       => $apvcReports,
		] );

	}

	/**
	 * Advanced Page Visit Counter Import Data from Page Visit Counter.
	 *
	 * @since    3.0.1
	 */
	public function apvc_import_from_pvc__premium_only() {

		if ( ! wp_verify_nonce( $_POST['security'], 'security_nonce' ) ) {
			die( 'Permission Denied.' );
		}

		global $wpdb;
		$tbl_history     = APVC_DATA_TABLE;
		$old_tbl_history = $wpdb->prefix . "page_visit_history";

		$existingData = get_option( "apvc_existing_ids" );

		$tempArray  = [];
		$tempCount  = 1;
		$allOldData = $wpdb->get_results( "SELECT * FROM $old_tbl_history" );
		foreach ( $allOldData as $data ):
			$exIP    = $data->ipaddress;
			$exArtID = $data->page_id;
			$exDate  = $data->date;
			if ( ! in_array( $data->id, $existingData ) ) {
				$last_id = $wpdb->insert( $tbl_history, [
					'article_id'         => $data->page_id,
					'date'               => $data->date,
					'last_date'          => $data->lastdate,
					'article_type'       => $data->type,
					'user_type'          => ( $data->user_role == 'guest' ) ? 'guest' : 'registered',
					'device_type'        => '',
					'ip_address'         => $data->ipaddress,
					'user_id'            => $data->user_id,
					'browser_full_name'  => $data->browser_full_name,
					'browser_short_name' => $data->browser_short_name,
					'browser_version'    => $data->browser_version,
					'operating_system'   => $data->os,
					'http_referer'       => $data->http_referer,
					'site_id'            => 1,
					'country'            => $data->country,
				], [
					'%d',
					'%s',
					'%s',
					'%s',
					'%s',
					'%s',
					'%s',
					'%d',
					'%s',
					'%s',
					'%s',
					'%s',
					'%s',
					'%d',
					'%s',
				] );
				array_push( $tempArray, $data->id );
			}
			if ( $tempCount % 100 == 0 ) {
				sleep( 2 );
			}
			$tempCount ++;
		endforeach;
		if ( count( $tempArray ) > 0 ) {
			update_option( "apvc_existing_ids", $tempArray );
		}
		echo wp_json_encode( [ "message" => "success" ] );
		wp_die();
	}

	/**
	 * Advanced Page Visit Counter Upload file.
	 *
	 * @since    3.0.1
	 */
	public function apvc_upload_file__premium_only() {

		if ( ! wp_verify_nonce( $_POST['security'], 'security_nonce' ) ) {
			die( 'Permission Denied.' );
		}

		foreach ( $_FILES as $file ) :
			if ( $file['error'] == UPLOAD_ERR_NO_FILE ) :
				continue;
			endif;

			$valid_ext        = [ 'csv', 'xls', 'xlsx' ];
			$extension_upload = strtolower( substr( strrchr( $file['name'], '.' ), 1 ) );
			if ( in_array( $extension_upload, $valid_ext ) ) :
				$url_insert = APVC_IMPORT_PATH . '/' . $file['name'];
				$action     = move_uploaded_file( $file['tmp_name'], $url_insert );
				$msg_true   = 'Upload ok';
			else :
				$msg_error = 'Upload error';
			endif;
		endforeach;

		$result = ! isset( $msg_error );
		$msg    = [];

		if ( $result ) :
			$msg['error'] = 'true';
			$msg['true']  = $msg_true;
			$msg['file']  = $file['name'];
		else :
			$msg['error'] = 'false';
			$msg['false'] = $msg_error;
		endif;

		header( 'Content-Type: application/json' );
		echo json_encode( $msg );
		exit();
	}

	/**
	 * Advanced Page Visit Counter Get import files data.
	 *
	 * @since    3.0.1
	 */
	public function apvc_import_file_data__premium_only() {

		if ( ! wp_verify_nonce( $_POST['security'], 'security_nonce' ) ) {
			die( 'Permission Denied.' );
		}

		global $wpdb;
		$tbl_history   = APVC_DATA_TABLE;
		$tbl_locations = APVC_USER_TABLE;

		$_file = sanitize_file_name( $_REQUEST['file'] );

		if ( $this->apvc_version_check__premium_only() == TRUE ) {
			$default_array = [
				'id',
				'article_id',
				'article_type',
				'user_type',
				'device_type',
				'date',
				'last_date',
				'ip_address',
				'browser_full_name',
				'browser_short_name',
				'browser_version',
				'operating_system',
				'http_referer',
				'user_id',
				'site_id',
				'flag',
				'country',
				"city",
				"state",
				"lsc_country",
				"country_code",
				"continent",
				"continent_code",
			];
			// 23
		} else {
			$default_array = [
				'id',
				'article_id',
				'article_type',
				'user_type',
				'device_type',
				'date',
				'last_date',
				'ip_address',
				'browser_full_name',
				'browser_short_name',
				'browser_version',
				'operating_system',
				'http_referer',
				'user_id',
				'site_id',
				'flag',
				'country',
			];
			// 17
		}

		$rowArray = [];
		if ( ( $handle = fopen( APVC_IMPORT_PATH . "/" . $_file, "r" ) ) !== FALSE ) {
			while ( ( $data = fgetcsv( $handle, 1000, "," ) ) !== FALSE ) {
				$num = count( $data );
				for ( $c = 0; $c < $num; $c ++ ) {
					array_push( $rowArray, trim( $data[ $c ] ) );
				}
				break;
			}
			fclose( $handle );
		}


		$CR = array_diff( $rowArray, $default_array );
		if ( count( $CR ) == 0 ) {
			$rowArray = [];
			if ( ( $handle = fopen( APVC_IMPORT_PATH . "/" . $_file, "r" ) ) !== FALSE ) {
				$rwCount = 0;
				while ( ( $data = fgetcsv( $handle, 1000, "," ) ) !== FALSE ) {
					$num       = count( $data );
					$tempArray = [];
					for ( $c = 0; $c < $num; $c ++ ) {
						array_push( $tempArray, trim( $data[ $c ] ) );
					}
					array_push( $rowArray, $tempArray );
					$rwCount ++;
				}
				fclose( $handle );
			}

			$wait = 1;
			for ( $rowCnt = 1; $rowCnt <= count( $rowArray ); $rowCnt ++ ) {
				$checkVal = $wpdb->get_var( "SELECT count(id) FROM {$tbl_history} WHERE date='" . $rowArray[ $rowCnt ][5] . "' AND article_id=" . $rowArray[ $rowCnt ][1] . "" );
				if ( $checkVal <= 0 ) {
					if ( count( $rowArray[ $rowCnt ] ) == 23 ) {
						$wpdb->insert( $tbl_history, [
							'article_id'         => $rowArray[ $rowCnt ][1],
							'date'               => $rowArray[ $rowCnt ][5],
							'last_date'          => $rowArray[ $rowCnt ][6],
							'article_type'       => $rowArray[ $rowCnt ][2],
							'user_type'          => $rowArray[ $rowCnt ][3],
							'device_type'        => $rowArray[ $rowCnt ][4],
							'ip_address'         => $rowArray[ $rowCnt ][7],
							'user_id'            => $rowArray[ $rowCnt ][13],
							'browser_full_name'  => $rowArray[ $rowCnt ][8],
							'browser_short_name' => $rowArray[ $rowCnt ][9],
							'browser_version'    => $rowArray[ $rowCnt ][10],
							'operating_system'   => $rowArray[ $rowCnt ][11],
							'http_referer'       => $rowArray[ $rowCnt ][12],
							'site_id'            => $rowArray[ $rowCnt ][14],
							'country'            => $rowArray[ $rowCnt ][16],
						], [
							'%d',
							'%s',
							'%s',
							'%s',
							'%s',
							'%s',
							'%s',
							'%d',
							'%s',
							'%s',
							'%s',
							'%s',
							'%s',
							'%d',
							'%s',
						] );

						if ( ! empty( $rowArray[ $rowCnt ][18] ) ) {
							$wpdb->insert( $tbl_locations, [
								'ent_id'         => $wpdb->insert_id,
								'city'           => $rowArray[ $rowCnt ][17],
								'state'          => $rowArray[ $rowCnt ][18],
								'country'        => $rowArray[ $rowCnt ][19],
								'country_code'   => $rowArray[ $rowCnt ][20],
								'continent'      => $rowArray[ $rowCnt ][21],
								'continent_code' => $rowArray[ $rowCnt ][22],
							], [ '%d', '%s', '%s', '%s', '%s', '%s', '%s' ] );
						}
					} else {
						$wpdb->insert( $tbl_history, [
							'article_id'         => $rowArray[ $rowCnt ][1],
							'date'               => $rowArray[ $rowCnt ][5],
							'last_date'          => $rowArray[ $rowCnt ][6],
							'article_type'       => $rowArray[ $rowCnt ][2],
							'user_type'          => $rowArray[ $rowCnt ][3],
							'device_type'        => $rowArray[ $rowCnt ][4],
							'ip_address'         => $rowArray[ $rowCnt ][7],
							'user_id'            => $rowArray[ $rowCnt ][13],
							'browser_full_name'  => $rowArray[ $rowCnt ][8],
							'browser_short_name' => $rowArray[ $rowCnt ][9],
							'browser_version'    => $rowArray[ $rowCnt ][10],
							'operating_system'   => $rowArray[ $rowCnt ][11],
							'http_referer'       => $rowArray[ $rowCnt ][12],
							'site_id'            => $rowArray[ $rowCnt ][14],
							'country'            => $rowArray[ $rowCnt ][16],
						], [
							'%d',
							'%s',
							'%s',
							'%s',
							'%s',
							'%s',
							'%s',
							'%d',
							'%s',
							'%s',
							'%s',
							'%s',
							'%s',
							'%d',
							'%s',
						] );
					}

				}
				if ( ( $wait % 100 ) == 0 ) {
					sleep( 1 );
				}
				$wait ++;
			}
			echo wp_json_encode( [ 'message' => 'success' ] );
		} else {
			echo wp_json_encode( [ 'message' => 'failed' ] );
		}
		wp_die();
	}


	/**
	 * Advanced Page Visit Counter Export Data CSV.
	 *
	 * @since    3.0.1
	 */
	public function apvc_export_data__premium_only() {
		global $wpdb;
		$tbl_history   = APVC_DATA_TABLE;
		$export_table  = APVC_EXPORT_TABLE;
		$tbl_locations = APVC_USER_TABLE;

		$_action  = (isset($_REQUEST['s_action'])) ? sanitize_text_field( $_REQUEST['s_action'] ) : "";
		$_sDate   = (isset($_REQUEST['sDate'])) ? sanitize_text_field( $_REQUEST['sDate'] ) : "";
		$_eDate   = (isset($_REQUEST['eDate'])) ? sanitize_text_field( $_REQUEST['eDate'] ) : "";
		$_data    = (isset($_REQUEST['a_data'])) ? sanitize_text_field( $_REQUEST['a_data'] ) : [];
		$fileName = '';
		$title    = '';

		if ( ! empty( $_sDate ) && ! empty( $_eDate ) ) {
			$dateCondition = " AND date >= '$_sDate' AND date <= '$_eDate' ";
		} else if ( ! empty( $_sDate ) ) {
			$dateCondition = " AND date >= '$_sDate' AND date <= '$_sDate' ";
		} else if ( ! empty( $_eDate ) ) {
			$dateCondition = " AND date <= '$_eDate' AND date >= '$_eDate' ";
		} else {
			$dateCondition = '';
		}
		if ( $_action == 'by_ptypes' ) {
			$aType = implode( "','", $_data );
			if ( $this->apvc_version_check__premium_only() == TRUE ) {
				$exData = $wpdb->get_results( "SELECT $tbl_history.*, $tbl_locations.city, $tbl_locations.state, $tbl_locations.country as lc_country, $tbl_locations.country_code, $tbl_locations.continent, $tbl_locations.continent_code FROM {$tbl_history} LEFT JOIN $tbl_locations ON $tbl_history.id = $tbl_locations.ent_id WHERE article_type IN('$aType') " . $dateCondition );
			} else {
				$exData = $wpdb->get_results( "SELECT * FROM {$tbl_history} WHERE article_type IN('$aType') " . $dateCondition );
			}
			$fileName = "post_types_export_";
			$title    = 'Custom Post Type Export';
		} else if ( $_action == 'by_pages' ) {
			$aPType = implode( "','", $_data );
			if ( $this->apvc_version_check__premium_only() == TRUE ) {
				$exData = $wpdb->get_results( "SELECT $tbl_history.*, $tbl_locations.city, $tbl_locations.state, $tbl_locations.country as lc_country, $tbl_locations.country_code, $tbl_locations.continent, $tbl_locations.continent_code FROM {$tbl_history} LEFT JOIN $tbl_locations ON $tbl_history.id = $tbl_locations.ent_id WHERE article_id IN('$aPType') " . $dateCondition );
			} else {
				$exData = $wpdb->get_results( "SELECT * FROM {$tbl_history} WHERE article_id IN('$aPType') " . $dateCondition );
			}
			$fileName = "pages_posts_export_";
			$title    = 'Pages/Posts Export';
		} else if ( $_action == 'by_country' ) {
			$aCountryType = implode( "','", $_data );
			if ( $this->apvc_version_check__premium_only() == TRUE ) {
				$exData = $wpdb->get_results( "SELECT $tbl_history.*, $tbl_locations.city, $tbl_locations.state, $tbl_locations.country as lc_country, $tbl_locations.country_code, $tbl_locations.continent, $tbl_locations.continent_code FROM {$tbl_history} LEFT JOIN $tbl_locations ON $tbl_history.id = $tbl_locations.ent_id WHERE country IN('$aCountryType') " . $dateCondition );
			} else {
				$exData = $wpdb->get_results( "SELECT * FROM {$tbl_history} WHERE country IN('$aCountryType') " . $dateCondition );
			}
			$fileName = "country_export_";
			$title    = 'Country Export';
		} else if ( $_action == 'by_referrer' ) {
			$aRefType = implode( "','", $_data );
			if ( $this->apvc_version_check__premium_only() == TRUE ) {
				$exData = $wpdb->get_results( "SELECT $tbl_history.*, $tbl_locations.city, $tbl_locations.state, $tbl_locations.country as lc_country, $tbl_locations.country_code, $tbl_locations.continent, $tbl_locations.continent_code FROM {$tbl_history} LEFT JOIN $tbl_locations ON $tbl_history.id = $tbl_locations.ent_id WHERE http_referer LIKE '%$aRefType%' " . $dateCondition );
			} else {
				$exData = $wpdb->get_results( "SELECT * FROM {$tbl_history} WHERE http_referer LIKE '%$aRefType%' " . $dateCondition );
			}
			$fileName = "referrer_export_";
			$title    = 'Referrer Export';
		} else if ( $_action == 'by_all' ) {
			$dateCondition = substr( $dateCondition, 4 );
			if ( $dateCondition != '' ) {
				if ( $this->apvc_version_check__premium_only() == TRUE ) {
					$exData = $wpdb->get_results( "SELECT $tbl_history.*, $tbl_locations.city, $tbl_locations.state, $tbl_locations.country as lc_country, $tbl_locations.country_code, $tbl_locations.continent, $tbl_locations.continent_code  FROM {$tbl_history} LEFT JOIN $tbl_locations ON $tbl_history.id = $tbl_locations.ent_id WHERE " . $dateCondition );
				} else {
					$exData = $wpdb->get_results( "SELECT * FROM {$tbl_history} WHERE " . $dateCondition );
				}
			} else {
				if ( $this->apvc_version_check__premium_only() == TRUE ) {
					$exData = $wpdb->get_results( "SELECT $tbl_history.*, $tbl_locations.city, $tbl_locations.state, $tbl_locations.country as lc_country, $tbl_locations.country_code, $tbl_locations.continent, $tbl_locations.continent_code  FROM {$tbl_history} LEFT JOIN $tbl_locations ON $tbl_history.id = $tbl_locations.ent_id" );
				} else {
					$exData = $wpdb->get_results( "SELECT * FROM {$tbl_history}" );
				}
			}
			$fileName = "all_export_";
			$title    = 'All Data Export';
		}

		$fileName    .= date( 'Y-m-d-H-i' );
		$csv_handler = fopen( APVC_EXPORT_PATH . "/" . $fileName . ".csv", 'w' );

		if ( $this->apvc_version_check__premium_only() == TRUE ) {
			$csv = [
				"id",
				"article_id",
				"article_type",
				"user_type",
				"device_type",
				"date",
				"last_date",
				"ip_address",
				"browser_full_name",
				"browser_short_name",
				"browser_version",
				"operating_system",
				"http_referer",
				"user_id",
				"site_id",
				"flag",
				"country",
				"city",
				"state",
				"lsc_country",
				"country_code",
				"continent",
				"continent_code",
			];
		} else {
			$csv = [
				"id",
				"article_id",
				"article_type",
				"user_type",
				"device_type",
				"date",
				"last_date",
				"ip_address",
				"browser_full_name",
				"browser_short_name",
				"browser_version",
				"operating_system",
				"http_referer",
				"user_id",
				"site_id",
				"flag",
				"country",
			];
		}

		fputcsv( $csv_handler, $csv );
		foreach ( $exData as $eData ) {
			if ( $this->apvc_version_check__premium_only() == TRUE ) {
				fputcsv( $csv_handler, [
					"id"                 => $eData->id,
					"article_id"         => $eData->article_id,
					"article_type"       => $eData->article_type,
					"user_type"          => $eData->user_type,
					"device_type"        => $eData->device_type,
					"date"               => $eData->date,
					"last_date"          => $eData->last_date,
					"ip_address"         => $eData->ip_address,
					"browser_full_name"  => $eData->browser_full_name,
					"browser_short_name" => $eData->browser_short_name,
					"browser_version"    => $eData->browser_version,
					"operating_system"   => $eData->operating_system,
					"http_referer"       => $eData->http_referer,
					"user_id"            => $eData->user_id,
					"site_id"            => $eData->site_id,
					"flag"               => $eData->flag,
					"country"            => $eData->country,
					"city"               => $eData->city,
					"state"              => $eData->state,
					"lc_country"         => $eData->lc_country,
					"country_code"       => $eData->country_code,
					"continent"          => $eData->continent,
					"continent_code"     => $eData->continent_code,
				] );
			} else {
				fputcsv( $csv_handler, [
					"id"                 => $eData->id,
					"article_id"         => $eData->article_id,
					"article_type"       => $eData->article_type,
					"user_type"          => $eData->user_type,
					"device_type"        => $eData->device_type,
					"date"               => $eData->date,
					"last_date"          => $eData->last_date,
					"ip_address"         => $eData->ip_address,
					"browser_full_name"  => $eData->browser_full_name,
					"browser_short_name" => $eData->browser_short_name,
					"browser_version"    => $eData->browser_version,
					"operating_system"   => $eData->operating_system,
					"http_referer"       => $eData->http_referer,
					"user_id"            => $eData->user_id,
					"site_id"            => $eData->site_id,
					"flag"               => $eData->flag,
					"country"            => $eData->country,
				] );
			}
		}
		fclose( $csv_handler );

		if ( file_exists( APVC_EXPORT_PATH . "/" . $fileName . ".csv" ) ) {
			$last_id = $wpdb->insert( $export_table, [
				'file_name'     => $fileName,
				'title'         => $title,
				'file_location' => APVC_EXPORT_PATH . "/" . $fileName . ".csv",
				'date'          => current_time( 'mysql' ),
			], [ '%s', '%s', '%s', '%s' ] );

			$fileURL = get_home_url() . "/wp-content/uploads/apvc_export/" . $fileName . ".csv";
			if ( $last_id != "" ) {
				echo wp_json_encode( [
					"message" => "success",
					"file"    => $fileURL,
				] );
			} else {
				echo wp_json_encode( [ "message" => "failed", "file" => "" ] );
			}
		}
		wp_die();
	}


	/**
	 * Advanced Page Visit Counter checks which version is installed and based
	 * on that it returns if the version is latest or not.
	 *
	 * @since    5.0.2
	 */
	public function apvc_version_check__premium_only() {
		if ( intval( str_replace( ".", "", ADVANCED_PAGE_VISIT_COUNTER ) ) >= 500 ) {
			return TRUE;
		} else {
			return FALSE;
		}
	}

	/**
	 * Advanced Page Visit Counter Clean Up records from the Database.
	 *
	 * @since    3.0.1
	 */
	public function apvc_cleanup_data__premium_only() {
		global $wpdb;
		$tbl_history = APVC_DATA_TABLE;

		$_action  = (isset($_REQUEST['s_action'])) ? sanitize_text_field( $_REQUEST['s_action'] ) : "";
		$_sDate   = (isset($_REQUEST['sDate'])) ? sanitize_text_field( $_REQUEST['sDate'] ) : "";
		$_eDate   = (isset($_REQUEST['eDate'])) ? sanitize_text_field( $_REQUEST['eDate'] ) : "";
		$_data    = (isset($_REQUEST['a_data'])) ? sanitize_text_field( $_REQUEST['a_data'] ) : [];
		$fileName = '';
		$title    = '';

		if ( ! empty( $_sDate ) && ! empty( $_eDate ) ) {
			$dateCondition = " AND date(date) >= '$_sDate' AND date(date) <= '$_eDate' ";
		} else if ( ! empty( $_sDate ) ) {
			$dateCondition = " AND date(date) >= '$_sDate' AND date(date) <= '$_sDate' ";
		} else if ( ! empty( $_eDate ) ) {
			$dateCondition = " AND date(date) <= '$_eDate' AND date(date) <= '$_eDate' ";
		} else {
			$dateCondition = '';
		}

		if ( $_action == 'by_ptypes' ) {
			$aType  = implode( "','", $_data );
			$exData = $wpdb->query( "DELETE FROM {$tbl_history} WHERE article_type IN('$aType') " . $dateCondition );
		} else if ( $_action == 'by_pages' ) {
			$aPType = implode( "','", $_data );
			$exData = $wpdb->query( "DELETE FROM {$tbl_history} WHERE article_id IN('$aPType') " . $dateCondition );
		} else if ( $_action == 'by_country' ) {
			$aCountryType = implode( "','", $_data );
			$exData       = $wpdb->query( "DELETE FROM {$tbl_history} WHERE country IN('$aCountryType') " . $dateCondition );
		} else if ( $_action == 'by_referrer' ) {
			$aRefType = implode( "','", $_data );
			$exData   = $wpdb->query( "DELETE FROM {$tbl_history} WHERE http_referer LIKE '%$aRefType%' " . $dateCondition );
		} else if ( $_action == 'by_all' ) {
			$exData   = $wpdb->query( "DELETE FROM {$tbl_history} WHERE article_id != '' " . $dateCondition );
		}	


		foreach ( $this->transients as $transient ) :
			delete_transient( $transient );
		endforeach;

		echo wp_json_encode( [ "message" => "success" ] );
		wp_die();
	}

	/**
	 * Advanced Page Visit Counter DELETE File.
	 *
	 * @since    3.0.1
	 */
	public function apvc_delete_file__premium_only() {

		if ( ! wp_verify_nonce( $_POST['security'], 'security_nonce' ) ) {
			die( 'Permission Denied.' );
		}

		global $wpdb;
		$tbl_export = APVC_EXPORT_TABLE;
		$_flID      = sanitize_text_field( $_REQUEST['flID'] );

		$filePath = $wpdb->get_var( "SELECT file_location FROM {$tbl_export} WHERE id=$_flID" );
		unlink( $filePath );
		$wpdb->query( "DELETE FROM {$tbl_export} WHERE id=$_flID" );

		echo 'success';
		wp_die();
	}

	/**
	 * Advanced Page Visit Counter Get posts pages data.
	 *
	 * @since    3.0.1
	 */
	public function apvc_get_posts_pages_data__premium_only() {
		global $wpdb;
		$tbl_history = APVC_DATA_TABLE;
		$allArticles = $wpdb->get_results( "SELECT article_id, ( SELECT post_title FROM {$wpdb->prefix}posts WHERE ID=article_id ) as title FROM {$tbl_history} GROUP BY article_id" );

		return wp_json_encode( $allArticles );
		wp_die();
	}

	/**
	 * Advanced Page Visit Counter Get countries data.
	 *
	 * @since    3.0.1
	 */
	public function apvc_get_countries_data__premium_only() {
		global $wpdb;
		$tbl_history  = APVC_DATA_TABLE;
		$allCountries = $wpdb->get_results( "SELECT country FROM {$tbl_history} GROUP BY country" );

		return wp_json_encode( $allCountries );
		wp_die();
	}

	/**
	 * Advanced Page Visit Counter Get referrers data.
	 *
	 * @since    3.0.1
	 */
	public function apvc_get_referrers_data__premium_only() {
		global $wpdb;
		$tbl_history   = APVC_DATA_TABLE;
		$referralStats = $wpdb->get_results( "SELECT SUBSTRING_INDEX(SUBSTRING_INDEX(REPLACE(REPLACE(LOWER(http_referer), 'https://', ''), 'http://', ''), '/', 1), '?', 1) as http_referer FROM {$tbl_history} GROUP BY http_referer", ARRAY_N );

		$result = [];
		foreach ( $referralStats as $key => $value ) {
			array_push( $result, $value[0] );
		}

		return wp_json_encode( array_unique( $result ) );
		wp_die();
	}


	/**
	 * Advanced Page Visit Counter Get countries data.
	 *
	 * @since    3.0.1
	 */
	public function apvc_get_web_crawlers__premium_only() {

		$crawlers = explode( "|", esc_html( "rambler|abacho|acoi|accona|aspseek|altavista|estyle|scrubby|lycos|geona|ia_archiver|alexa|sogou|skype|facebook|twitter|pinterest|linkedin|naver|bing|google|yahoo|duckduckgo|yandex|baidu|teoma|xing|java 1.7.0_45|bot|crawl|slurp|spider|mediapartners|sasks|saols|dumbot" ) );

		return wp_json_encode( $crawlers );
	}

	/**
	 * Advanced Page Visit Counter Get icons.
	 *
	 * @since    3.0.1
	 */
	public function apvc_get_icons() {

		$icons = [
			'None',
			'icon-eye',
			'icon-user',
			'icon-people',
			'icon-user-female',
			'icon-user-follow',
			'icon-user-following',
			'icon-emotsmile',
			'icon-location-pin',
			'icon-list',
			'icon-options',
			'icon-clock',
			'icon-plus',
			'icon-trophy',
			'icon-screen-desktop',
			'icon-plane',
			'icon-mouse',
			'icon-mustache',
			'icon-cursor-move',
			'icon-cursor',
			'icon-energy',
			'icon-screen-tablet',
			'icon-shield',
			'icon-speedometer',
			'icon-chemistry',
			'icon-magic-wand',
			'icon-disc',
			'icon-graduation',
			'icon-ghost',
			'icon-eyeglass',
			'icon-fire',
			'icon-bell',
			'icon-game-controller',
			'icon-speech',
			'icon-badge',
			'icon-pin',
			'icon-playlist',
			'icon-present',
			'icon-picture',
			'icon-globe',
			'icon-diamond',
			'icon-basket-loaded',
			'icon-cup',
			'icon-rocket',
			'icon-home',
			'icon-music-tone-alt',
			'icon-music-tone',
			'icon-earphones-alt',
			'icon-graph',
			'icon-microphone',
			'icon-control-play',
			'icon-calendar',
			'icon-bulb',
			'icon-chart',
			'icon-camera',
			'icon-cloud-download',
			'icon-bubble',
			'icon-heart',
			'icon-star',
		];

		return wp_json_encode( $icons );
	}

	public function apvc_sanitize_text_string( $input ) {
		// Remove "type=" and any attributes after it
		$sanitized_input = preg_replace('/\btype\s*=\s*("[^"]*"|\'[^\']*\'|[^"\'>\s]*)/i', '', $input);
		$pattern = '/type=image\s+src=1\s+onerror=alert\(1\)>/i';
		$sanitized_input = preg_replace($pattern, '', $sanitized_input);

		// Use str_replace to remove the specific string and variations
		$sanitized_input = str_replace(
			['type=image src=1 onerror=alert(1)>', 'type=image src=1 onerror=alert(1)&gt;', 'type=image src=1 onerror=alert(1)&gt;'],
			'',
			$sanitized_input
		);

		$sanitized_input = str_replace(
			['src=1 onerror=alert(1)>', 'type=image src=1 onerror=alert(1)&gt;', 'type=image src=1 onerror=alert(1)&gt;'],
			'',
			$sanitized_input
		);

		$sanitized_input = str_replace(
			['onerror=alert(1)>', 'type=image src=1 onerror=alert(1)&gt;', 'type=image src=1 onerror=alert(1)&gt;'],
			'',
			$sanitized_input
		);

		// Optionally, trim any leading or trailing whitespace
		$sanitized_input = trim($sanitized_input);

		return sanitize_text_field( $sanitized_input );
	}

	/**
	 * Advanced Page Visit Counter Save settings method.
	 *
	 * @since    3.0.1
	 */
	public function apvc_save_settings() {

		if ( ! wp_verify_nonce( $_POST['security'], 'security_nonce' ) ) {
			die( 'Permission Denied.' );
		}

		if ( ! current_user_can( 'manage_options' ) ) {
			echo 'failed';
			exit;
		}

		$formData = $this->apvc_sanitize_text_string( $_POST['formData'] );
		$default_label = $this->apvc_sanitize_text_string( $_POST['default_label'] );
		$todays_label = $this->apvc_sanitize_text_string( $_POST['todays_label'] );
		$global_label = $this->apvc_sanitize_text_string( $_POST['global_label'] );
		$range_posts = $this->apvc_sanitize_text_string( $_POST['range_posts'] );
		$range_ip_add = $this->apvc_sanitize_text_string( $_POST['range_ip_add'] );
		$ex_show_counter = $this->apvc_sanitize_text_string( $_POST['ex_show_counter'] );
		$formDataStr   = $formData;
		$formData      = explode( "&", $formData );
		$finalFormData = [];

		if ( strpos( $formDataStr, 'cache_active' ) ) {
			update_option( 'cache_active', 'Yes' );
		} else {
			update_option( 'cache_active', 'No' );
		}
		if ( strpos( $formDataStr, 'numbers_in_k' ) ) {
			update_option( 'numbers_in_k', 'Yes' );
		} else {
			update_option( 'numbers_in_k', 'No' );
		}
		foreach ( $formData as $key => $value ):
			$rawFormData = explode( "=", $value );
			if ( isset( $rawFormData[0] ) ):
				if( 'apvc_default_label' === $rawFormData[0] ){
					$finalFormData[ $rawFormData[0] ][] = sanitize_text_field($default_label);
				} else if( 'apvc_todays_label' === $rawFormData[0] ){
					$finalFormData[ $rawFormData[0] ][] = sanitize_text_field($todays_label);
				} else if( 'apvc_global_label' === $rawFormData[0] ){
					$finalFormData[ $rawFormData[0] ][] = sanitize_text_field($global_label);
				} else if( 'apvc_exclude_counts' === $rawFormData[0] ){
					$finalFormData[ $rawFormData[0] ][] = $range_posts;
				} else if( 'apvc_ip_address' === $rawFormData[0] ){
					$finalFormData[ $rawFormData[0] ][] = $range_ip_add;
				} else if( 'apvc_exclude_show_counter' === $rawFormData[0] ){
					$finalFormData[ $rawFormData[0] ][] = $ex_show_counter;
				} else {
					$finalFormData[ $rawFormData[0] ][] = sanitize_text_field($rawFormData[1]);
				}
			endif;

		endforeach;

		update_option( "apvc_configurations", $finalFormData );
		echo "success";

		wp_die();
	}

	/**
	 * Advanced Page Visit Counter Reset settings method.
	 *
	 * @since    3.0.1
	 */
	public function apvc_reset_settings() {

		if ( ! wp_verify_nonce( $_POST['security'], 'security_nonce' ) ) {
			die( 'Permission Denied.' );
		}

		$finalFormData                                  = [];
		$finalFormData['apvc_post_types']               = [ 'post', 'page' ];
		$finalFormData['apvc_widget_width']             = [ 300 ];
		$finalFormData['apvc_default_label']            = [ 'Visits' ];
		$finalFormData['apvc_wid_alignment']            = [ 'center' ];
		$finalFormData['apvc_default_text_color']       = [ '#000000' ];
		$finalFormData['apvc_default_background_color'] = [ '#fffffff' ];
		$finalFormData['apvc_default_border_color']     = [ '#000000' ];
		$finalFormData['apvc_default_border_radius']    = [ 5 ];
		$finalFormData['apvc_default_border_width']     = [ 2 ];
		update_option( "apvc_configurations", $finalFormData );
		echo "success";
		wp_die();
	}

	/**
	 * Advanced Page Visit Counter Reset data method.
	 *
	 * @since    3.0.1
	 */
	public function apvc_reset_data() {

		if ( ! wp_verify_nonce( $_POST['security'], 'security_nonce' ) ) {
			die( 'Permission Denied.' );
		}

		global $wpdb;
		$tbl_history = APVC_DATA_TABLE;
		$wpdb->query( "TRUNCATE TABLE {$tbl_history}" );
		$wpdb->query( "DELETE FROM $wpdb->postmeta WHERE meta_key='count_start_from'" );

		$tbl_users = APVC_USER_TABLE;
		$wpdb->query( "TRUNCATE TABLE {$tbl_users}" );

		echo "success";
		wp_die();
	}

	/**
	 * Advanced Page Visit Counter Edit Shortcode Generator Method.
	 *
	 * @since    3.0.1
	 */
	public function apvc_edit_shortcode__premium_only() {

		if ( ! wp_verify_nonce( $_POST['security'], 'security_nonce' ) ) {
			die( 'Permission Denied.' );
		}

		global $wpdb;
		$tbl_shortcodes = APVC_SHORTCODES;
		$_shID          = sanitize_text_field( $_REQUEST['shID'] );

		$sh            = $wpdb->get_var( "SELECT config FROM {$tbl_shortcodes} WHERE id=$_shID" );
		$title         = $wpdb->get_var( "SELECT title FROM {$tbl_shortcodes} WHERE id=$_shID" );
		$formData      = $sh;
		$formData      = explode( "&", $formData );
		$finalFormData = [];

		foreach ( $formData as $key => $value ):
			$rawFormData = explode( "=", $value );

			if ( isset( $rawFormData[0] ) ):
				$finalFormData[ $rawFormData[0] ][] = urldecode( trim( sanitize_text_field( $rawFormData[1] ) ) );
			endif;

		endforeach;
		echo wp_json_encode( [
			"formData" => $finalFormData,
			"title"    => $title,
		] );
		wp_die();
	}

	/**
	 * Advanced Page Visit Counter Show Shortcode Generator Method.
	 *
	 * @since    3.0.1
	 */
	public function apvc_show_shortcode__premium_only() {

		if ( ! wp_verify_nonce( $_POST['security'], 'security_nonce' ) ) {
			die( 'Permission Denied.' );
		}

		global $wpdb;
		ob_start();
		$tbl_shortcodes = APVC_SHORTCODES;
		$_shID          = sanitize_text_field( $_REQUEST['shID'] );

		$sh = $wpdb->get_var( "SELECT shortcode FROM {$tbl_shortcodes} WHERE id=$_shID" );
		echo '<div id="shCopy">' . esc_html( $sh ) . '</div>';
		echo '<br />';
		echo '<div class="text-center">';
		echo do_shortcode( esc_html( $sh ) );
		echo '</div>';
		echo ob_get_clean();
		wp_die();
	}

	/**
	 * Advanced Page Visit Counter Delete Shortcode Generator Method.
	 *
	 * @since    3.0.1
	 */
	public function apvc_delete_shortcode__premium_only() {

		if ( ! wp_verify_nonce( $_POST['security'], 'security_nonce' ) ) {
			die( 'Permission Denied.' );
		}

		global $wpdb;
		$tbl_shortcodes = APVC_SHORTCODES;
		$_shID          = sanitize_text_field( $_REQUEST['shID'] );
		$wpdb->query( "DELETE FROM {$tbl_shortcodes} WHERE id=$_shID" );
		echo 'success';
		wp_die();
	}

	/**
	 * Advanced Page Visit Counter Save Shortcode Generator Method.
	 *
	 * @since    3.0.1
	 */
	public function apvc_save_shortcode__premium_only() {

		if ( ! wp_verify_nonce( $_POST['security'], 'security_nonce' ) ) {
			die( 'Permission Denied.' );
		}

		global $wpdb;
		$tbl_shortcodes = APVC_SHORTCODES;

		$title     = sanitize_text_field( $_REQUEST['title'] );
		$formData  = sanitize_text_field( $_REQUEST['formData'] );
		$shortcode = sanitize_text_field( $_REQUEST['shortcode'] );
		$last_id   = $wpdb->insert( $tbl_shortcodes, [
			'title'     => $title,
			'shortcode' => $shortcode,
			'config'    => $formData,
			'date'      => current_time( 'mysql' ),
		], [ '%s', '%s', '%s', '%s' ] );

		if ( $last_id != "" ) {
			echo wp_json_encode( [ "message" => "success" ] );
		} else {
			echo wp_json_encode( [ "message" => "failed" ] );
		}
		wp_die();
	}

	/**
	 * Advanced Page Visit Counter Get All Articles.
	 *
	 * @since    3.0.1
	 */
	public function apvc_get_all_articles_sh() {

		if ( ! wp_verify_nonce( $_POST['security'], 'security_nonce' ) ) {
			die( 'Permission Denied.' );
		}

		global $wpdb;
		$allArticles = $wpdb->get_results( "SELECT ID, post_title FROM $wpdb->posts WHERE post_type NOT IN('attachment','revision','nav_menu_item') AND post_status='publish'" );

		$array = [];
		$html  = '';
		foreach ( $allArticles as $article ):
			$html .= '<option value="' . $article->ID . '">' . $article->post_title . '</option>';
		endforeach;

		ob_clean();
		wp_send_json_success($html) ;
		wp_die();
	}


	/**
	 * Get column head title in wordpress admin
	 *
	 * @since    1.0.0
	 *
	 * @return  string
	 */
	public function apvc_columns_label( $defaults ) {

		$defaults['views'] = esc_html__( 'Views', "advanced-page-visit-counter" );

		return $defaults;
	}

	/**
	 * Get column head content in wordpress admin
	 *
	 * @param string $column_id
	 * @param int $article_id
	 *
	 * @since    3.0.6
	 *
	 */
	public function apvc_columns_counts( $column_id, $article_id ) {
		global $wpdb;
		$tbl_history = APVC_DATA_TABLE;

		if ( $column_id == 'views' ) {

			global $wpdb;
			$post_result = get_post( $article_id );

			if ( 'post' === $post_result->post_type ) {
				$type = 'post';
			} else {
				$type = 'page';
			}

			$totalVisits = $wpdb->get_var( "SELECT count(*) FROM " . APVC_DATA_TABLE . " WHERE article_id = {$article_id}" );

			if ( $totalVisits ) {
				echo '<a href="' . esc_url( site_url( '/wp-admin/admin.php?page=apvc-dashboard-page&apvc_page=detailed-reports&article_id=' . esc_attr( $article_id ) ) ) . '">' . esc_html( $totalVisits ) . '</a>';
			} else {
				echo "0";
			}

		}

	}

	/**
	 * Advanced Page Visit Counter Save posts meta for counter.
	 *
	 * @since    3.0.1
	 */
	public function apvc_advanced_save_metaboxes( $post_id ) {
		global $wpdb;
		$tbl_history = APVC_DATA_TABLE;

		$active_count = ( isset($_POST["apvc_active_counter"]) ) ? sanitize_text_field($_POST["apvc_active_counter"]) : "";
	    $reset_count = ( isset($_POST["apvc_reset_cnt"]) ) ? sanitize_text_field($_POST["apvc_reset_cnt"]) : "";
	    $start_count = ( isset($_POST["count_start_from"]) ) ? sanitize_text_field($_POST["count_start_from"]) : "";
	    $widget_label = ( isset($_POST["widget_label"]) ) ? sanitize_text_field($_POST["widget_label"]) : "";

		if ( empty( $active_count ) ) {
			$active_count = "Yes";
		}

		update_post_meta( $post_id, "apvc_active_counter", $active_count );
		update_post_meta( $post_id, "count_start_from", $start_count );
		update_post_meta( $post_id, "widget_label", $widget_label );

		if ( $reset_count == "Yes" ) {
			$wpdb->query( "DELETE FROM $tbl_history WHERE article_id=$post_id" );
		}
	}

	/**
	 * Advanced Page Visit Counter Upgrade process method.
	 *
	 * @since    3.0.1
	 */
	public function apvc_upgrader_process_complete() {

		set_transient( "apvcSetPromo", "no", 86400 );
		update_option( 'apvc_promo', 'yes' );
		$current_version = get_option( "apvc_version" );
		// $this->apvc_migrate_from_old_version($current_version);
		update_option( "apvc_version", ADVANCED_PAGE_VISIT_COUNTER );
		update_option( "apvc_newsletter", "show" );
		update_option( 'apvc_notice', 'no' );
	}

	public function apvc_get_html_with_icon( $class ) {
		return '<div class="' . $class . '"><div><i class="icon-graph icons"></i> Visits: <span>999</span></div><div><i class="icon-eyeglass icons"></i> Today: <span>123</span></div><div><i class="icon-chart icons"></i> Total: <span>123</span></div></div>';
	}

	public function apvc_get_html_without_icon( $class ) {

		return '<div class="' . $class . '"><div>Visits: <span>999</span></div><div>Today: <span>123</span></div><div>Total: <span>123</span></div></div>';
	}

	/**
	 * Advanced Page Visit Counter Shortcode Library.
	 *
	 * @since    3.0.1
	 */
	public function apvc_get_shortcodes( $shortcode = '' ) {

		$shortcodes = [];

		$shortcodes['template_3']['name'] = __('Template 3', 'advanced-page-visit-counter');
		$shortcodes['template_3']['key'] = 'template_3';
		$shortcodes['template_3']['icon'] = 'yes';
		$shortcodes['template_3']['css']  = '.template_3{background:#1c8394;padding:15px;margin:15px;border-radius:50px;border:2px solid #1c8394;-webkit-box-shadow:3px 4px 12px -2px rgba(0,0,0,.68);-moz-box-shadow:3px 4px 12px -2px rgba(0,0,0,.68);box-shadow:3px 4px 12px -2px rgba(0,0,0,.68);font-family:calibri;font-size:13pt;text-align:center}.template_3>div{color:#fff;display:inline-block;margin:0 30px}.template_3>div>span{font-weight:700;margin-left:10px}.template_3 .icons{color:#fff;margin-right:5px;font-weight:700}@media (max-width:644px){.template_3>div{margin:0 10px}}@media (max-width:525px){.template_3>div{color:#fff;display:block;margin:0;padding:10px 0;border-bottom:1px solid #fff}.template_3>div:last-child{border-bottom:none}}';

		$shortcodes['template_6']['name'] = __('Template 6', 'advanced-page-visit-counter');
		$shortcodes['template_6']['key'] = 'template_6';
		$shortcodes['template_6']['icon']  = 'yes';
		$shortcodes['template_6']['class'] = 'effect2';
		$shortcodes['template_6']['css']   = '.template_6{background:#764ba2;background:linear-gradient(90deg,#667eea 0,#764ba2 100%);padding:15px;margin:15px;border-radius:40px;border:2px solid #764ba2;font-family:calibri;font-size:13pt;text-align:center}.effect2{position:relative}.effect2:after{z-index:-1;position:absolute;content:"";bottom:15px;right:10px;left:auto;width:50%;top:50%;max-width:300px;background:#777;-webkit-box-shadow:0 15px 10px #777;-moz-box-shadow:0 15px 10px #777;box-shadow:0 15px 10px #777;-webkit-transform:rotate(4deg);-moz-transform:rotate(4deg);-o-transform:rotate(4deg);-ms-transform:rotate(4deg);transform:rotate(4deg)}.template_6>div{color:#fff;display:inline-block;margin:0 30px}.template_6>div>span{font-weight:700;margin-left:10px}.template_6 .icons{color:#fff;margin-right:5px;font-weight:700}@media (max-width:644px){.template_6>div{margin:0 10px}}@media (max-width:525px){.template_6>div{display:block;margin:0;padding:10px 0;border-bottom:1px solid #fcb8a1}.template_6>div:last-child{border-bottom:none}}';

		$shortcodes['template_7']['name'] = __('Template 7', 'advanced-page-visit-counter');
		$shortcodes['template_7']['key'] = 'template_7';
		$shortcodes['template_7']['icon']  = 'yes';
		$shortcodes['template_7']['class'] = 'effect2';
		$shortcodes['template_7']['css']   = '.template_7{background:#dfa579;background:linear-gradient(90deg,#c79081 0,#dfa579 100%);padding:15px;margin:15px;border-radius:40px;border:2px solid #dfa579;font-family:calibri;font-size:13pt;text-align:center}.effect2{position:relative}.effect2:after,.effect2:before{z-index:-1;position:absolute;content:"";bottom:25px;left:10px;width:50%;top:35%;max-width:300px;background:#000;-webkit-box-shadow:0 35px 20px #000;-moz-box-shadow:0 35px 20px #000;box-shadow:0 35px 20px #000;-webkit-transform:rotate(-7deg);-moz-transform:rotate(-7deg);-o-transform:rotate(-7deg);-ms-transform:rotate(-7deg);transform:rotate(-7deg)}.effect2:after{-webkit-transform:rotate(7deg);-moz-transform:rotate(7deg);-o-transform:rotate(7deg);-ms-transform:rotate(7deg);transform:rotate(7deg);right:10px;left:auto}.template_7>div{color:#fff;display:inline-block;margin:0 30px}.template_7>div>span{font-weight:700;margin-left:10px}.template_7 .icons{color:#fff;margin-right:5px;font-weight:700}@media (max-width:644px){.template_7>div{margin:0 10px}}@media (max-width:525px){.template_7>div{display:block;margin:0;padding:10px 0;border-bottom:1px solid #fcb8a1}.template_7>div:last-child{border-bottom:none}}';

		$shortcodes['template_8']['name'] = __('Template 8', 'advanced-page-visit-counter');
		$shortcodes['template_8']['key'] = 'template_8';
		$shortcodes['template_8']['icon']  = 'yes';
		$shortcodes['template_8']['class'] = 'effect2';
		$shortcodes['template_8']['css']   = '.template_8{background:#5fc3e4;background:linear-gradient(90deg,#e55d87 0,#5fc3e4 100%);padding:15px;margin:15px;border:2px solid #5fc3e4;font-family:calibri;font-size:13pt;text-align:center}.effect2{position:relative;-webkit-box-shadow:0 1px 4px rgba(0,0,0,.3),0 0 40px rgba(0,0,0,.1) inset;-moz-box-shadow:0 1px 4px rgba(0,0,0,.3),0 0 40px rgba(0,0,0,.1) inset;box-shadow:0 1px 4px rgba(0,0,0,.3),0 0 40px rgba(0,0,0,.1) inset}.effect2:after,.effect2:before{content:"";position:absolute;z-index:-1;-webkit-box-shadow:0 0 20px rgba(0,0,0,.8);-moz-box-shadow:0 0 20px rgba(0,0,0,.8);box-shadow:0 0 20px rgba(0,0,0,.8);top:0;bottom:0;left:10px;right:10px;-moz-border-radius:100px/10px;border-radius:100px/10px}.effect2:after{right:10px;left:auto;-webkit-transform:skew(8deg) rotate(3deg);-moz-transform:skew(8deg) rotate(3deg);-ms-transform:skew(8deg) rotate(3deg);-o-transform:skew(8deg) rotate(3deg);transform:skew(8deg) rotate(3deg)}.template_8>div{color:#fff;display:inline-block;margin:0 30px}.template_8>div>span{font-weight:700;margin-left:10px}.template_8 .icons{color:#fff;margin-right:5px;font-weight:700}@media (max-width:644px){.template_8>div{margin:0 10px}}@media (max-width:525px){.template_8>div{display:block;margin:0;padding:10px 0;border-bottom:1px solid #fff}.template_8>div:last-child{border-bottom:none}}';

		$shortcodes['template_11']['name'] = __('Template 11', 'advanced-page-visit-counter');
		$shortcodes['template_11']['key'] = 'template_11';
		$shortcodes['template_11']['icon'] = 'yes';
		$shortcodes['template_11']['css']  = '.template_11{background:#2980b9;background:linear-gradient(225deg,#2980b9 0,#6dd5fa 50%,#fff 100%);padding:15px;margin:15px;border-radius:40px;border:2px solid #2980b9;font-family:calibri;font-size:13pt;text-align:center}.template_11>div{color:#1a1a1a;display:inline-block;margin:0 30px}.template_11>div>span{font-weight:700;margin-left:10px}.template_11 .icons{color:#1a1a1a;margin-right:5px;font-weight:700}@media (max-width:644px){.template_11>div{margin:0 10px}}@media (max-width:525px){.template_11>div{display:block;margin:0;padding:10px 0;border-bottom:1px solid #2980b9}.template_11>div:last-child{border-bottom:none}}';

		$shortcodes['template_22']['name'] = __('Template 22', 'advanced-page-visit-counter');
		$shortcodes['template_22']['key'] = 'template_22';
		$shortcodes['template_22']['icon'] = 'no';
		$shortcodes['template_22']['css']  = '.template_22{background:#355c7d;background:linear-gradient(90deg,#355c7d 0,#6c5b7b 50%,#c06c84 100%);padding:15px;margin:15px;font-family:calibri;font-size:13pt;text-align:center;-webkit-box-shadow:0 10px 14px 0 rgba(0,0,0,.1);-moz-box-shadow:0 10px 14px 0 rgba(0,0,0,.1);box-shadow:0 10px 14px 0 rgba(0,0,0,.1)}.template_22>div{color:#fff;display:inline-block;margin:0 30px}.template_22>div>span{font-weight:700;margin-left:10px}@media (max-width:644px){.template_22>div{margin:0 10px}}@media (max-width:525px){.template_22>div{display:block;margin:0;padding:10px 0;border-bottom:1px solid #c06c84}.template_22>div:last-child{border-bottom:none}}';

		$shortcodes['template_23']['name'] = __('Template 23', 'advanced-page-visit-counter');
		$shortcodes['template_23']['key'] = 'template_23';
		$shortcodes['template_23']['icon'] = 'no';
		$shortcodes['template_23']['css']  = '.template_23{background:#fc5c7d;background:linear-gradient(90deg,#fc5c7d 0,#6c5b7b 50%,#6a82fb 100%);padding:15px;margin:15px;font-family:calibri;font-size:13pt;text-align:center;-webkit-box-shadow:0 10px 14px 0 rgba(0,0,0,.1);-moz-box-shadow:0 10px 14px 0 rgba(0,0,0,.1);box-shadow:0 10px 14px 0 rgba(0,0,0,.1)}.template_23>div{color:#fff;display:inline-block;margin:0 30px}.template_23>div>span{font-weight:700;margin-left:10px}@media (max-width:644px){.template_23>div{margin:0 10px}}@media (max-width:525px){.template_23>div{display:block;margin:0;padding:10px 0;border-bottom:1px solid #c06c84}.template_23>div:last-child{border-bottom:none}}';

		$shortcodes['template_24']['name'] = __('Template 24', 'advanced-page-visit-counter');
		$shortcodes['template_24']['key'] = 'template_24';
		$shortcodes['template_24']['icon'] = 'no';
		$shortcodes['template_24']['css']  = '.template_24{background:#fffbd5;background:linear-gradient(90deg,#fffbd5 0,#b20a2c 50%);padding:15px;margin:15px;font-family:calibri;font-size:13pt;text-align:center;-webkit-box-shadow:0 10px 14px 0 rgba(0,0,0,.1);-moz-box-shadow:0 10px 14px 0 rgba(0,0,0,.1);box-shadow:0 10px 14px 0 rgba(0,0,0,.1)}.template_24>div{color:#fff;display:inline-block;margin:0 30px}.template_24>div>span{font-weight:700;margin-left:10px}@media (max-width:644px){.template_24>div{margin:0 10px}}@media (max-width:525px){.template_24>div{display:block;margin:0;padding:10px 0;border-bottom:1px solid #fffbd5}.template_24>div:last-child{border-bottom:none}}';

		$shortcodes['template_25']['name'] = __('Template 25', 'advanced-page-visit-counter');
		$shortcodes['template_25']['key'] = 'template_25';
		$shortcodes['template_25']['icon'] = 'no';
		$shortcodes['template_25']['css']  = '.template_25{background:#302b63;background:linear-gradient(90deg,#0f0c29 0,#7365ff 50%,#24243e 100%);padding:15px;margin:15px;font-family:calibri;font-size:13pt;text-align:center;-webkit-box-shadow:0 10px 14px 0 rgba(0,0,0,.1);-moz-box-shadow:0 10px 14px 0 rgba(0,0,0,.1);box-shadow:0 10px 14px 0 rgba(0,0,0,.1)}.template_25>div{color:#fff;display:inline-block;margin:0 30px}.template_25>div>span{font-weight:700;margin-left:10px}@media (max-width:644px){.template_25>div{margin:0 10px}}@media (max-width:525px){.template_25>div{display:block;margin:0;padding:10px 0;border-bottom:1px solid #0f0c29}.template_25>div:last-child{border-bottom:none}}';

		$shortcodes['template_26']['name'] = __('Template 26', 'advanced-page-visit-counter');
		$shortcodes['template_26']['key'] = 'template_26';
		$shortcodes['template_26']['icon'] = 'no';
		$shortcodes['template_26']['css']  = '.template_26{background:#d3cce3;background:linear-gradient(90deg,#d3cce3 0,#e9e4f0 50%,#d3cce3 100%);padding:15px;margin:15px;font-family:calibri;font-size:13pt;text-align:center;-webkit-box-shadow:0 10px 14px 0 rgba(0,0,0,.1);-moz-box-shadow:0 10px 14px 0 rgba(0,0,0,.1);box-shadow:0 10px 14px 0 rgba(0,0,0,.1)}.template_26>div{color:#6a6279;display:inline-block;margin:0 30px}.template_26>div>span{font-weight:700;margin-left:10px}@media (max-width:644px){.template_26>div{margin:0 10px}}@media (max-width:525px){.template_26>div{display:block;margin:0;padding:10px 0;border-bottom:1px solid #7f7a8a}.template_26>div:last-child{border-bottom:none}}';

		$shortcodes['template_29']['name'] = __('Template 29', 'advanced-page-visit-counter');
		$shortcodes['template_29']['key'] = 'template_29';
		$shortcodes['template_29']['icon'] = 'no';
		$shortcodes['template_29']['css']  = '.template_29{background:#6d6027;background:linear-gradient(90deg,#6d6027 0,#d3cbb8 80%,#3c3b3f 100%);padding:15px;margin:15px;font-family:calibri;font-size:13pt;text-align:center;-webkit-box-shadow:0 10px 14px 0 rgba(0,0,0,.2);-moz-box-shadow:0 10px 14px 0 rgba(0,0,0,.2);box-shadow:0 10px 14px 0 rgba(0,0,0,.2)}.template_29>div{color:#fff;display:inline-block;margin:0 30px}.template_29>div>span{font-weight:700;margin-left:10px}@media (max-width:644px){.template_29>div{margin:0 10px}}@media (max-width:525px){.template_29>div{display:block;margin:0;padding:10px 0;border-bottom:1px solid #00f260}.template_29>div:last-child{border-bottom:none}}';

		$shortcodes['template_31']['name'] = __('Template 31', 'advanced-page-visit-counter');
		$shortcodes['template_31']['key'] = 'template_31';
		$shortcodes['template_31']['icon'] = 'no';
		$shortcodes['template_31']['css']  = '.template_31{background:#3a1c71;background:linear-gradient(90deg,#3a1c71 0,#d76d77 25%,#ffaf7b 50%);padding:15px;margin:15px;font-family:calibri;font-size:13pt;text-align:center;-webkit-box-shadow:0 10px 14px 0 rgba(0,0,0,.2);-moz-box-shadow:0 10px 14px 0 rgba(0,0,0,.2);box-shadow:0 10px 14px 0 rgba(0,0,0,.2)}.template_31>div{color:#1a1a1a;display:inline-block;margin:0 30px}.template_31>div>span{font-weight:700;margin-left:10px}@media (max-width:644px){.template_31>div{margin:0 10px}}@media (max-width:525px){.template_31>div{display:block;margin:0;padding:10px 0;border-bottom:1px solid #fff}.template_31>div:last-child{border-bottom:none}}';

		$shortcodes['template_34']['name'] = __('Template 34', 'advanced-page-visit-counter');
		$shortcodes['template_34']['key'] = 'template_34';
		$shortcodes['template_34']['icon'] = 'no';
		$shortcodes['template_34']['css']  = '.template_34{background:#f7971e;background:linear-gradient(90deg,#f7971e 0,#ffd200 50%,#f7971e 1%);padding:15px;margin:15px;font-family:calibri;font-size:13pt;text-align:center;-webkit-box-shadow:0 10px 14px 0 rgba(0,0,0,.2);-moz-box-shadow:0 10px 14px 0 rgba(0,0,0,.2);box-shadow:0 10px 14px 0 rgba(0,0,0,.2)}.template_34>div{color:#1a1a1a;display:inline-block;margin:0 30px}.template_34>div>span{font-weight:700;margin-left:10px}@media (max-width:644px){.template_34>div{margin:0 10px}}@media (max-width:525px){.template_34>div{display:block;margin:0;padding:10px 0;border-bottom:1px solid #fff}.template_34>div:last-child{border-bottom:none}}';

		$shortcodes['template_39']['name'] = __('Template 39', 'advanced-page-visit-counter');
		$shortcodes['template_39']['key'] = 'template_39';
		$shortcodes['template_39']['icon'] = 'no';
		$shortcodes['template_39']['css']  = '.template_39{background:#000;background:linear-gradient(90deg,#000 0,#b3cc2c 50%);padding:15px;margin:15px;font-family:calibri;font-size:13pt;text-align:center;-webkit-box-shadow:0 10px 14px 0 rgba(0,0,0,.2);-moz-box-shadow:0 10px 14px 0 rgba(0,0,0,.2);box-shadow:0 10px 14px 0 rgba(0,0,0,.2)}.template_39>div{color:#fff;display:inline-block;margin:0 30px}.template_39>div>span{font-weight:700;margin-left:10px}@media (max-width:644px){.template_39>div{margin:0 10px}}@media (max-width:525px){.template_39>div{display:block;margin:0;padding:10px 0;border-bottom:1px solid #fff}.template_39>div:last-child{border-bottom:none}}';

		$shortcodes['template_40']['name'] = __('Template 40', 'advanced-page-visit-counter');
		$shortcodes['template_40']['key'] = 'template_40';
		$shortcodes['template_40']['icon'] = 'no';
		$shortcodes['template_40']['css']  = '.template_40{background:#ba8b02;background:linear-gradient(90deg,#ba8b02 0,#ffd65d 80%,#ba8b02 100%);padding:15px;margin:15px;font-family:calibri;font-size:13pt;text-align:center;-webkit-box-shadow:0 10px 14px 0 rgba(0,0,0,.2);-moz-box-shadow:0 10px 14px 0 rgba(0,0,0,.2);box-shadow:0 10px 14px 0 rgba(0,0,0,.2)}.template_40>div{color:#1a1a1a;display:inline-block;margin:0 30px}.template_40>div>span{font-weight:700;margin-left:10px}@media (max-width:644px){.template_40>div{margin:0 10px}}@media (max-width:525px){.template_40>div{display:block;margin:0;padding:10px 0;border-bottom:1px solid #fff}.template_40>div:last-child{border-bottom:none}}';

		if ( apvc_fs()->is__premium_only() ) {

			$shortcodes['template_1']['name'] = __('Template 1', 'advanced-page-visit-counter');
			$shortcodes['template_1']['key'] = 'template_1';
			$shortcodes['template_1']['icon'] = 'yes';
			$shortcodes['template_1']['css']  = '.template_1{background:#4e1c94;background:linear-gradient(90deg,#4e1c94 0,#d4480c 100%);padding:15px;margin:15px;border-top-left-radius:20px;border-bottom-right-radius:20px;border:2px solid #a52e06;-webkit-box-shadow:3px 4px 12px -2px rgba(0,0,0,.38);-moz-box-shadow:3px 4px 12px -2px rgba(0,0,0,.38);box-shadow:3px 4px 12px -2px rgba(0,0,0,.38);font-family:calibri;font-size:13pt;text-align:center}.template_1>div{color:#fff;display:inline-block;margin:0 30px}.template_1>div>span{font-weight:700;margin-left:10px}.template_1 .icons{color:#fff;margin-right:5px;font-weight:700}@media (max-width:644px){.template_1>div{margin:0 10px}}@media (max-width:525px){.template_1>div{color:#fff;display:block;margin:0;padding:10px 0;border-bottom:1px solid #ca4516}.template_1>div:last-child{border-bottom:none}}';

			$shortcodes['template_2']['name'] = __('Template 2', 'advanced-page-visit-counter');
			$shortcodes['template_2']['key'] = 'template_2';
			$shortcodes['template_2']['icon'] = 'yes';
			$shortcodes['template_2']['css']  = '.template_2{background:#1c8394;background:linear-gradient(90deg,#1c8394 0,#c2d40c 100%);padding:15px;margin:15px;border:2px solid #1c8394;-webkit-box-shadow:3px 4px 12px -2px rgba(0,0,0,.68);-moz-box-shadow:3px 4px 12px -2px rgba(0,0,0,.68);box-shadow:3px 4px 12px -2px rgba(0,0,0,.68);font-family:calibri;font-size:13pt;text-align:center}.template_2>div{color:#fff;display:inline-block;margin:0 30px}.template_2>div>span{font-weight:700;margin-left:10px}.template_2 .icons{color:#fff;margin-right:5px;font-weight:700}@media (max-width:644px){.template_2>div{margin:0 10px}}@media (max-width:525px){.template_2>div{color:#fff;display:block;margin:0;padding:10px 0;border-bottom:1px solid #1c8394}.template_2>div:last-child{border-bottom:none}}';

			$shortcodes['template_4']['name'] = __('Template 4', 'advanced-page-visit-counter');
			$shortcodes['template_4']['key'] = 'template_4';
			$shortcodes['template_4']['icon'] = 'yes';
			$shortcodes['template_4']['css']  = '.template_4{background:#faffd1;background:linear-gradient(90deg,#a1ffce 0,#faffd1 100%);padding:15px;margin:15px;border-top-left-radius:20px;border-top-right-radius:20px;border:2px solid #d6de95;-webkit-box-shadow:0 8px 6px -6px #000;-moz-box-shadow:0 8px 6px -6px #000;box-shadow:0 8px 6px -6px #000;font-family:calibri;font-size:13pt;text-align:center}.template_4>div{color:#1a1a1a;display:inline-block;margin:0 30px}.template_4>div>span{font-weight:700;margin-left:10px}.icons{color:#1a1a1a;margin-right:5px;font-weight:700}@media (max-width:644px){.template_4>div{margin:0 10px}}@media (max-width:525px){.template_4>div{display:block;margin:0;padding:10px 0;border-bottom:1px solid #d6de95}.template_4>div:last-child{border-bottom:none}}';

			$shortcodes['template_5']['name'] = __('Template 5', 'advanced-page-visit-counter');
			$shortcodes['template_5']['key'] = 'template_5';
			$shortcodes['template_5']['icon']  = 'yes';
			$shortcodes['template_5']['class'] = 'effect2';
			$shortcodes['template_5']['css']   = '.template_5{background:#fcb69f;background:linear-gradient(90deg,#ffecd2 0,#fcb69f 100%);padding:15px;margin:15px;border-top-left-radius:20px;border-top-right-radius:20px;border:2px solid #fcb8a1;font-family:calibri;font-size:13pt;text-align:center}.effect2{position:relative}.effect2:after,.effect2:before{z-index:-1;position:absolute;content:"";bottom:15px;left:10px;width:50%;top:60%;max-width:300px;background:#777;-webkit-box-shadow:0 15px 10px #777;-moz-box-shadow:0 15px 10px #777;box-shadow:0 15px 10px #777;-webkit-transform:rotate(-3deg);-moz-transform:rotate(-3deg);-o-transform:rotate(-3deg);-ms-transform:rotate(-3deg);transform:rotate(-3deg)}.effect2:after{-webkit-transform:rotate(3deg);-moz-transform:rotate(3deg);-o-transform:rotate(3deg);-ms-transform:rotate(3deg);transform:rotate(3deg);right:10px;left:auto}.template_5>div{color:#1a1a1a;display:inline-block;margin:0 30px}.template_5>div>span{font-weight:700;margin-left:10px}.template_5 .icons{color:#1a1a1a;margin-right:5px;font-weight:700}@media (max-width:644px){.template_5>div{margin:0 10px}}@media (max-width:525px){.template_5>div{display:block;margin:0;padding:10px 0;border-bottom:1px solid #fcb8a1}.template_5>div:last-child{border-bottom:none}}';

			$shortcodes['template_9']['name'] = __('Template 9', 'advanced-page-visit-counter');
			$shortcodes['template_9']['key'] = 'template_9';
			$shortcodes['template_9']['icon'] = 'yes';
			$shortcodes['template_9']['css']  = '.template_9{background:#009fe4;background:linear-gradient(90deg,#0f2027 0,#009fe4 100%);padding:15px;margin:15px;border:2px solid #5fc3e4;font-family:calibri;border-bottom-right-radius:40px;font-size:13pt;text-align:center;box-shadow:5px 5px rgba(0,159,228,.4),10px 10px rgba(0,159,228,.3),15px 15px rgba(0,159,228,.2),20px 20px rgba(0,159,228,.1),25px 25px rgba(0,159,228,.05)}.template_9>div{color:#fff;display:inline-block;margin:0 30px}.template_9>div>span{font-weight:700;margin-left:10px}.template_9 .icons{color:#fff;margin-right:5px;font-weight:700}@media (max-width:644px){.template_9>div{margin:0 10px}}@media (max-width:525px){.template_9>div{display:block;margin:0;padding:10px 0;border-bottom:1px solid #fff}.template_9>div:last-child{border-bottom:none}}';

			$shortcodes['template_10']['name'] = __('Template 10', 'advanced-page-visit-counter');
			$shortcodes['template_10']['key'] = 'template_10';
			$shortcodes['template_10']['icon'] = 'yes';
			$shortcodes['template_10']['css']  = '.template_10{background:#c6ffdd;background:linear-gradient(90deg,#c6ffdd 0,#fbd786 50%,#f7797d 100%);padding:15px;margin:15px;border:2px solid #5fc3e4;font-family:calibri;font-size:13pt;text-align:center;-moz-transform:skew(1deg,1deg);-webkit-transform:skew(1deg,1deg);-o-transform:skew(1deg,1deg);-ms-transform:skew(1deg,1deg);transform:skew(1deg,1deg)}.template_10>div{color:#1a1a1a;display:inline-block;margin:0 30px}.template_10>div>span{font-weight:700;margin-left:10px}.template_10 .icons{color:#1a1a1a;margin-right:5px;font-weight:700}@media (max-width:644px){.template_10>div{margin:0 10px}}@media (max-width:525px){.template_10>div{display:block;margin:0;padding:10px 0;border-bottom:1px solid #fff}.template_10>div:last-child{border-bottom:none}}';

			$shortcodes['template_12']['name'] = __('Template 12', 'advanced-page-visit-counter');
			$shortcodes['template_12']['key'] = 'template_12';
			$shortcodes['template_12']['icon'] = 'yes';
			$shortcodes['template_12']['css']  = '.template_12{background:#493240;background:linear-gradient(315deg,#493240 0,#f09 100%);padding:15px;margin:15px;border-bottom-left-radius:40px;border-bottom-right-radius:40px;font-family:calibri;font-size:13pt;text-align:center}.template_12>div{color:#fff;display:inline-block;margin:0 30px}.template_12>div>span{font-weight:700;margin-left:10px}.template_12 .icons{color:#fff;margin-right:5px;font-weight:700}@media (max-width:644px){.template_12>div{margin:0 10px}}@media (max-width:525px){.template_12>div{display:block;margin:0;padding:10px 0;border-bottom:1px solid #2980b9}.template_12>div:last-child{border-bottom:none}}';

			$shortcodes['template_13']['name'] = __('Template 13', 'advanced-page-visit-counter');
			$shortcodes['template_13']['key'] = 'template_13';
			$shortcodes['template_13']['icon'] = 'yes';
			$shortcodes['template_13']['css']  = '.template_13{background:#493240;background:linear-gradient(315deg,#8e2de2 0,#4a00e0 100%);padding:15px;margin:15px;border-bottom-left-radius:40px;border-top-left-radius:40px;font-family:calibri;font-size:13pt;text-align:center}.template_13>div{color:#fff;display:inline-block;margin:0 30px}.template_13>div>span{font-weight:700;margin-left:10px}.template_13 .icons{color:#fff;margin-right:5px;font-weight:700}@media (max-width:644px){.template_13>div{margin:0 10px}}@media (max-width:525px){.template_13>div{display:block;margin:0;padding:10px 0;border-bottom:1px solid #8e2de2}.template_13>div:last-child{border-bottom:none}}';

			$shortcodes['template_14']['name'] = __('Template 14', 'advanced-page-visit-counter');
			$shortcodes['template_14']['key'] = 'template_14';
			$shortcodes['template_14']['icon'] = 'yes';
			$shortcodes['template_14']['css']  = '.template_14{background:#99f2c8;background:linear-gradient(315deg,#99f2c8 0,#1f4037 100%);padding:15px;margin:15px;border-bottom-left-radius:40px;border-top-left-radius:40px;font-family:calibri;font-size:13pt;text-align:center;border:2px solid #99f2c8}.template_14>div{color:#fff;display:inline-block;margin:0 30px}.template_14>div>span{font-weight:700;margin-left:10px}.template_14 .icons{color:#fff;margin-right:5px;font-weight:700}@media (max-width:644px){.template_14>div{margin:0 10px}}@media (max-width:525px){.template_14>div{display:block;margin:0;padding:10px 0;border-bottom:1px solid #99f2c8}.template_14>div:last-child{border-bottom:none}}';

			$shortcodes['template_15']['name'] = __('Template 15', 'advanced-page-visit-counter');
			$shortcodes['template_15']['key'] = 'template_15';
			$shortcodes['template_15']['icon'] = 'yes';
			$shortcodes['template_15']['css']  = '.template_15{background:#f12711;background:linear-gradient(315deg,#f12711 0,#f5af19 100%);padding:15px;margin:15px;border-bottom-left-radius:40px;border-top-left-radius:40px;font-family:calibri;font-size:13pt;text-align:center;border:2px dashed #f5af19}.template_15>div{color:#fff;display:inline-block;margin:0 30px}.template_15>div>span{font-weight:700;margin-left:10px}.template_15 .icons{color:#fff;margin-right:5px;font-weight:700}@media (max-width:644px){.template_15>div{margin:0 10px}}@media (max-width:525px){.template_15>div{display:block;margin:0;padding:10px 0;border-bottom:1px dashed #99f2c8}.template_15>div:last-child{border-bottom:none}}';

			$shortcodes['template_16']['name'] = __('Template 16', 'advanced-page-visit-counter');
			$shortcodes['template_16']['key'] = 'template_16';
			$shortcodes['template_16']['icon'] = 'yes';
			$shortcodes['template_16']['css']  = '.template_16{background:#8360c3;background:linear-gradient(315deg,#8360c3 0,#2ebf91 100%);padding:15px;margin:15px;font-family:calibri;font-size:13pt;text-align:center;border:2px dashed #8360c3}.template_16>div{color:#fff;display:inline-block;margin:0 30px}.template_16>div>span{font-weight:700;margin-left:10px}.template_16 .icons{color:#fff;margin-right:5px;font-weight:700}@media (max-width:644px){.template_16>div{margin:0 10px}}@media (max-width:525px){.template_16>div{display:block;margin:0;padding:10px 0;border-bottom:1px dashed #99f2c8}.template_16>div:last-child{border-bottom:none}}';

			$shortcodes['template_17']['name'] = __('Template 17', 'advanced-page-visit-counter');
			$shortcodes['template_17']['key'] = 'template_17';
			$shortcodes['template_17']['icon'] = 'yes';
			$shortcodes['template_17']['css']  = '.template_17{background:#eaafc8;background:linear-gradient(0deg,#654ea3 0,#eaafc8 100%);padding:15px;margin:15px;font-family:calibri;font-size:13pt;text-align:center;border:2px dashed #eaafc8}.template_17>div{color:#fff;display:inline-block;margin:0 30px}.template_17>div>span{font-weight:700;margin-left:10px}.template_17 .icons{color:#fff;margin-right:5px;font-weight:700}@media (max-width:644px){.template_17>div{margin:0 10px}}@media (max-width:525px){.template_17>div{display:block;margin:0;padding:10px 0;border-bottom:1px dashed #eaafc8}.template_17>div:last-child{border-bottom:none}}';

			$shortcodes['template_18']['name'] = __('Template 18', 'advanced-page-visit-counter');
			$shortcodes['template_18']['key'] = 'template_18';
			$shortcodes['template_18']['icon'] = 'yes';
			$shortcodes['template_18']['css']  = '.template_18{background:#f27121;background:linear-gradient(315deg,#8a2387 0,#e94057 50%,#f27121 100%);padding:15px;margin:15px;font-family:calibri;font-size:13pt;text-align:center;border:2px dashed #f27121}.template_18>div{color:#fff;display:inline-block;margin:0 30px}.template_18>div>span{font-weight:700;margin-left:10px}.template_18 .icons{color:#fff;margin-right:5px;font-weight:700}@media (max-width:644px){.template_18>div{margin:0 10px}}@media (max-width:525px){.template_18>div{display:block;margin:0;padding:10px 0;border-bottom:1px dashed #eaafc8}.template_18>div:last-child{border-bottom:none}}';

			$shortcodes['template_19']['name'] = __('Template 19', 'advanced-page-visit-counter');
			$shortcodes['template_19']['key'] = 'template_19';
			$shortcodes['template_19']['icon'] = 'yes';
			$shortcodes['template_19']['css']  = '.template_19{background:#78ffd6;background:linear-gradient(315deg,#a8ff78 0,#78ffd6 100%);padding:15px;margin:15px;font-family:calibri;font-size:13pt;text-align:center;-webkit-box-shadow:0 10px 14px 0 rgba(0,0,0,.1);-moz-box-shadow:0 10px 14px 0 rgba(0,0,0,.1);box-shadow:0 10px 14px 0 rgba(0,0,0,.1)}.template_19>div{color:#1a1a1a;display:inline-block;margin:0 30px}.template_19>div>span{font-weight:700;margin-left:10px}.template_19 .icons{color:#1a1a1a;margin-right:5px;font-weight:700}@media (max-width:644px){.template_19>div{margin:0 10px}}@media (max-width:525px){.template_19>div{display:block;margin:0;padding:10px 0;border-bottom:1px solid #1a1a1a}.template_19>div:last-child{border-bottom:none}}';

			$shortcodes['template_20']['name'] = __('Template 20', 'advanced-page-visit-counter');
			$shortcodes['template_20']['key'] = 'template_20';
			$shortcodes['template_20']['icon'] = 'yes';
			$shortcodes['template_20']['css']  = '.template_20{background:#f37335;background:linear-gradient(90deg,#fdc830 0,#f37335 100%);padding:15px;margin:15px;font-family:calibri;font-size:13pt;text-align:center;-webkit-box-shadow:0 10px 14px 0 rgba(0,0,0,.1);-moz-box-shadow:0 10px 14px 0 rgba(0,0,0,.1);box-shadow:0 10px 14px 0 rgba(0,0,0,.1)}.template_20>div{color:#1a1a1a;display:inline-block;margin:0 30px}.template_20>div>span{font-weight:700;margin-left:10px}.template_20 .icons{color:#1a1a1a;margin-right:5px;font-weight:700}@media (max-width:644px){.template_20>div{margin:0 10px}}@media (max-width:525px){.template_20>div{display:block;margin:0;padding:10px 0;border-bottom:1px solid #1a1a1a}.template_20>div:last-child{border-bottom:none}}}';

			$shortcodes['template_21']['name'] = __('Template 21', 'advanced-page-visit-counter');
			$shortcodes['template_21']['key'] = 'template_21';
			$shortcodes['template_21']['icon'] = 'no';
			$shortcodes['template_21']['css']  = '.template_21{background:#333;background:linear-gradient(90deg,#333 0,#dd1818 100%);padding:15px;margin:15px;font-family:calibri;font-size:13pt;text-align:center;-webkit-box-shadow:0 10px 14px 0 rgba(0,0,0,.1);-moz-box-shadow:0 10px 14px 0 rgba(0,0,0,.1);box-shadow:0 10px 14px 0 rgba(0,0,0,.1)}.template_21>div{color:#fff;display:inline-block;margin:0 30px}.template_21>div>span{font-weight:700;margin-left:10px}@media (max-width:644px){.template_21>div{margin:0 10px}}@media (max-width:525px){.template_21>div{display:block;margin:0;padding:10px 0;border-bottom:1px solid #fff}.template_21>div:last-child{border-bottom:none}}';

			$shortcodes['template_27']['name'] = __('Template 27', 'advanced-page-visit-counter');
			$shortcodes['template_27']['key'] = 'template_27';
			$shortcodes['template_27']['icon'] = 'no';
			$shortcodes['template_27']['css']  = '.template_27{background:#3c3b3f;background:linear-gradient(90deg,#3c3b3f 0,#b3a84d 50%,#3c3b3f 100%);padding:15px;margin:15px;font-family:calibri;font-size:13pt;text-align:center;-webkit-box-shadow:0 10px 14px 0 rgba(0,0,0,.2);-moz-box-shadow:0 10px 14px 0 rgba(0,0,0,.2);box-shadow:0 10px 14px 0 rgba(0,0,0,.2)}.template_27>div{color:#fff;display:inline-block;margin:0 30px}.template_27>div>span{font-weight:700;margin-left:10px}@media (max-width:644px){.template_27>div{margin:0 10px}}@media (max-width:525px){.template_27>div{display:block;margin:0;padding:10px 0;border-bottom:1px solid #b3a84d}.template_27>div:last-child{border-bottom:none}}';

			$shortcodes['template_28']['name'] = __('Template 28', 'advanced-page-visit-counter');
			$shortcodes['template_28']['key'] = 'template_28';
			$shortcodes['template_28']['icon'] = 'no';
			$shortcodes['template_28']['css']  = '.template_28{background:#00f260;background:linear-gradient(90deg,#00f260 0,#0575e6 50%,#3c3b3f 100%);padding:15px;margin:15px;font-family:calibri;font-size:13pt;text-align:center;-webkit-box-shadow:0 10px 14px 0 rgba(0,0,0,.2);-moz-box-shadow:0 10px 14px 0 rgba(0,0,0,.2);box-shadow:0 10px 14px 0 rgba(0,0,0,.2)}.template_28>div{color:#fff;display:inline-block;margin:0 30px}.template_28>div>span{font-weight:700;margin-left:10px}@media (max-width:644px){.template_28>div{margin:0 10px}}@media (max-width:525px){.template_28>div{display:block;margin:0;padding:10px 0;border-bottom:1px solid #00f260}.template_28>div:last-child{border-bottom:none}}';


			$shortcodes['template_30']['name'] = __('Template 30', 'advanced-page-visit-counter');
			$shortcodes['template_30']['key'] = 'template_30';
			$shortcodes['template_30']['icon'] = 'no';
			$shortcodes['template_30']['css']  = '.template_30{background:#03001e;background:linear-gradient(90deg,#03001e 0,#7303c0 25%,#ec38bc 50%,#fdeff9 100%);padding:15px;margin:15px;font-family:calibri;font-size:13pt;text-align:center;-webkit-box-shadow:0 10px 14px 0 rgba(0,0,0,.2);-moz-box-shadow:0 10px 14px 0 rgba(0,0,0,.2);box-shadow:0 10px 14px 0 rgba(0,0,0,.2)}.template_30>div{color:#fff;display:inline-block;margin:0 30px}.template_30>div>span{font-weight:700;margin-left:10px}@media (max-width:644px){.template_30>div{margin:0 10px}}@media (max-width:525px){.template_30>div{display:block;margin:0;padding:10px 0;border-bottom:1px solid #fff}.template_30>div:last-child{border-bottom:none}}';


			$shortcodes['template_32']['name'] = __('Template 32', 'advanced-page-visit-counter');
			$shortcodes['template_32']['key'] = 'template_32';
			$shortcodes['template_32']['icon'] = 'no';
			$shortcodes['template_32']['css']  = '.template_32{background:#cbb4d4;background:linear-gradient(90deg,#20002c 0,#cbb4d4 100%);padding:15px;margin:15px;font-family:calibri;font-size:13pt;text-align:center;-webkit-box-shadow:0 10px 14px 0 rgba(0,0,0,.2);-moz-box-shadow:0 10px 14px 0 rgba(0,0,0,.2);box-shadow:0 10px 14px 0 rgba(0,0,0,.2)}.template_32>div{color:#fff;display:inline-block;margin:0 30px}.template_32>div>span{font-weight:700;margin-left:10px}@media (max-width:644px){.template_32>div{margin:0 10px}}@media (max-width:525px){.template_32>div{display:block;margin:0;padding:10px 0;border-bottom:1px solid #fff}.template_32>div:last-child{border-bottom:none}}';

			$shortcodes['template_33']['name'] = __('Template 33', 'advanced-page-visit-counter');
			$shortcodes['template_33']['key'] = 'template_33';
			$shortcodes['template_33']['icon'] = 'no';
			$shortcodes['template_33']['css']  = '.template_33{background:#eb5757;background:linear-gradient(90deg,#eb5757 0,#000 50%);padding:15px;margin:15px;font-family:calibri;font-size:13pt;text-align:center;-webkit-box-shadow:0 10px 14px 0 rgba(0,0,0,.2);-moz-box-shadow:0 10px 14px 0 rgba(0,0,0,.2);box-shadow:0 10px 14px 0 rgba(0,0,0,.2)}.template_33>div{color:#fff;display:inline-block;margin:0 30px}.template_33>div>span{font-weight:700;margin-left:10px}@media (max-width:644px){.template_33>div{margin:0 10px}}@media (max-width:525px){.template_33>div{display:block;margin:0;padding:10px 0;border-bottom:1px solid #fff}.template_33>div:last-child{border-bottom:none}}';


			$shortcodes['template_35']['name'] = __('Template 35', 'advanced-page-visit-counter');
			$shortcodes['template_35']['key'] = 'template_35';
			$shortcodes['template_35']['icon'] = 'no';
			$shortcodes['template_35']['css']  = '.template_35{background:#200122;background:linear-gradient(90deg,#200122 0,#6f0000 70%,#200122 0);padding:15px;margin:15px;font-family:calibri;font-size:13pt;text-align:center;-webkit-box-shadow:0 10px 14px 0 rgba(0,0,0,.2);-moz-box-shadow:0 10px 14px 0 rgba(0,0,0,.2);box-shadow:0 10px 14px 0 rgba(0,0,0,.2)}.template_35>div{color:#fff;display:inline-block;margin:0 30px}.template_35>div>span{font-weight:700;margin-left:10px}@media (max-width:644px){.template_35>div{margin:0 10px}}@media (max-width:525px){.template_35>div{display:block;margin:0;padding:10px 0;border-bottom:1px solid #fff}.template_35>div:last-child{border-bottom:none}}';

			$shortcodes['template_36']['name'] = __('Template 36', 'advanced-page-visit-counter');
			$shortcodes['template_36']['key'] = 'template_36';
			$shortcodes['template_36']['icon'] = 'no';
			$shortcodes['template_36']['css']  = '.template_36{background:#093028;background:linear-gradient(90deg,#093028 0,#237a57 30%,#093028 0);padding:15px;margin:15px;font-family:calibri;font-size:13pt;text-align:center;-webkit-box-shadow:0 10px 14px 0 rgba(0,0,0,.2);-moz-box-shadow:0 10px 14px 0 rgba(0,0,0,.2);box-shadow:0 10px 14px 0 rgba(0,0,0,.2)}.template_36>div{color:#fff;display:inline-block;margin:0 30px}.template_36>div>span{font-weight:700;margin-left:10px}@media (max-width:644px){.template_36>div{margin:0 10px}}@media (max-width:525px){.template_36>div{display:block;margin:0;padding:10px 0;border-bottom:1px solid #fff}.template_36>div:last-child{border-bottom:none}}';

			$shortcodes['template_37']['name'] = __('Template 37', 'advanced-page-visit-counter');
			$shortcodes['template_37']['key'] = 'template_37';
			$shortcodes['template_37']['icon'] = 'no';
			$shortcodes['template_37']['css']  = '.template_37{background:#dce35b;background:linear-gradient(90deg,#dce35b 0,#45b649 30%);padding:15px;margin:15px;font-family:calibri;font-size:13pt;text-align:center;-webkit-box-shadow:0 10px 14px 0 rgba(0,0,0,.2);-moz-box-shadow:0 10px 14px 0 rgba(0,0,0,.2);box-shadow:0 10px 14px 0 rgba(0,0,0,.2)}.template_37>div{color:#fff;display:inline-block;margin:0 30px}.template_37>div>span{font-weight:700;margin-left:10px}@media (max-width:644px){.template_37>div{margin:0 10px}}@media (max-width:525px){.template_37>div{display:block;margin:0;padding:10px 0;border-bottom:1px solid #fff}.template_37>div:last-child{border-bottom:none}}';

			$shortcodes['template_38']['name'] = __('Template 38', 'advanced-page-visit-counter');
			$shortcodes['template_38']['key'] = 'template_38';
			$shortcodes['template_38']['icon'] = 'no';
			$shortcodes['template_38']['css']  = '.template_38{background:#ffd89b;background:linear-gradient(90deg,#ffd89b 0,#19547b 50%);padding:15px;margin:15px;font-family:calibri;font-size:13pt;text-align:center;-webkit-box-shadow:0 10px 14px 0 rgba(0,0,0,.2);-moz-box-shadow:0 10px 14px 0 rgba(0,0,0,.2);box-shadow:0 10px 14px 0 rgba(0,0,0,.2)}.template_38>div{color:#fff;display:inline-block;margin:0 30px}.template_38>div>span{font-weight:700;margin-left:10px}@media (max-width:644px){.template_38>div{margin:0 10px}}@media (max-width:525px){.template_38>div{display:block;margin:0;padding:10px 0;border-bottom:1px solid #fff}.template_38>div:last-child{border-bottom:none}}';
		}

		if ( ! empty( $shortcode ) ) {
			return $shortcodes[ $shortcode ];
		} else {
			return $shortcodes;
		}
	}

	/**
	 * Advanced Page Visit Counter Migrate from older version.
	 *
	 * @since    3.0.1
	 */
	public function apvc_migrate_from_old_version( $version ) {
		global $wpdb;

		$version = get_option( "apvc_version" );
		if ( $version != '3.0.0' && $version != '3.0.1' && $version != '3.0.2' && $version != '3.0.3' && $version != '3.0.4' && $version != '3.0.5' && $version != '3.0.6' ) {
			$newConfArray  = [];
			$avc_configNew = (object) get_option( "apvc_configurations", TRUE );
			$avc_config    = json_decode( get_option( "avc_config", TRUE ) );

			foreach ( $avc_config as $key => $value ):

				if ( $key == 'post_types' ) {
					$newConfArray['apvc_post_types'] = $value;
				} else if ( $key == 'ip_address' ) {
					$ips = json_decode( $value );
					foreach ( $ips as $ip ):
						$newConfArray['apvc_ip_address'][] = $ip->tag;
					endforeach;
				} else if ( $key == 'exclude_counts' ) {
					$exCounts = json_decode( $value );
					foreach ( $exCounts as $exCount ):
						$newConfArray['apvc_exclude_counts'][] = $exCount->tag;
					endforeach;
				} else if ( $key == 'exclude_users' ) {
					$users = json_decode( $value );
					foreach ( $users as $user ):
						$newConfArray['apvc_exclude_users'][] = $user->tag;
					endforeach;
				} else if ( $key == 'exclude_show_counter' ) {
					$exSCounts = json_decode( $value );
					foreach ( $exSCounts as $exSCount ):
						$newConfArray['apvc_exclude_show_counter'][] = $exSCount->tag;
					endforeach;
				} else if ( $key == 'spam_controller' ) {
					$newConfArray['apvc_spam_controller'][0] = ( $value == "true" ) ? "on" : "";
				} else if ( $key == 'show_conter_on_fron_side' ) {
					$newConfArray['apvc_show_conter_on_front_side'][0] = $value;
				} else if ( $key == 'avc_default_text_color_of_counter' ) {
					$newConfArray['apvc_default_text_color'][0] = $value;
				} else if ( $key == 'apv_default_label' ) {
					$newConfArray['apvc_default_label'][0] = $value;
				} else if ( $key == 'apv_default_border_radius' ) {
					$newConfArray['apvc_default_border_radius'][0] = $value;
				} else if ( $key == 'apv_default_background_color' ) {
					$newConfArray['apvc_default_background_color'][0] = $value;
				} else if ( $key == 'apv_default_border_color' ) {
					$newConfArray['apvc_default_border_color'][0] = $value;
				} else if ( $key == 'apv_default_border_width' ) {
					$newConfArray['apvc_default_border_width'][0] = $value;
				} else if ( $key == 'wid_alignment' ) {
					$newConfArray['apvc_wid_alignment'][0] = $value;
				} else if ( $key == 'show_today_count' ) {
					$newConfArray['apvc_show_today_count'][0] = ( $value == "true" ) ? "on" : "";
				} else if ( $key == 'show_global_count' ) {
					$newConfArray['apvc_show_global_count'][0] = ( $value == "true" ) ? "on" : "";
				} else if ( $key == 'widget_width' ) {
					$newConfArray['apvc_widget_width'][0] = $value;
				}
			endforeach;
			update_option( "apvc_configurations", $newConfArray );
		}
	}


	public function apvc_get_visit_stats() {

		if ( ! wp_verify_nonce( $_POST['security'], 'security_nonce' ) ) {
			die( 'Permission Denied.' );
		}
		global $wpdb;
		$table = APVC_DATA_TABLE;

		$stats = get_transient( "apvc_get_visit_stats" );

		if ( empty( $stats ) ) {

			$labels = [];
			$dCount = [];
			$vCount = [];
			for ( $i = 0; $i < 20; $i ++ ) {
				$labels[] = date( "d-M", strtotime( '-' . $i . ' days' ) );
				$sDate    = date( "Y-m-d 0:0:0", strtotime( '-' . $i . ' days' ) );
				$eDate    = date( "Y-m-d 23:59:59", strtotime( '-' . $i . ' days' ) );

				$dCount[] = $wpdb->get_var( "SELECT COUNT(*) FROM {$table} WHERE date >= '$sDate' AND date <= '$eDate'" );

				$vCount[] = $wpdb->get_var( "SELECT COUNT(*) FROM {$table} WHERE date >= '$sDate' AND date <= '$eDate' GROUP BY ip_address" );
			}

			set_transient( "apvc_get_visit_stats", json_encode( [
				"labels"   => $labels,
				"visitors" => $vCount,
				"visits"   => $dCount,
			] ), APVC_HOURLY_REFRESH );
			echo wp_json_encode( [
				"message" => "Success",
				"chart"   => json_encode( [
					"labels"   => $labels,
					"visitors" => $vCount,
					"visits"   => $dCount,
				] ),
			] );
		} else {
			echo wp_json_encode( [
				"message" => "Success",
				"chart"   => $stats,
			] );
		}

		wp_die();
	}


	/**
	 * Advanced Page Visit Counter Get City States data for premium users only.
	 *
	 * @since    4.4.0
	 */
	public function apvc_get_city_states__premium_only( $ent_id ) {
		global $wpdb;

		if ( $ent_id == "" ) {
			return;
		}

		$tbl_city_states = $wpdb->prefix . "apvc_user_locations";
		$data            = $wpdb->get_results( "SELECT city,state FROM {$tbl_city_states} WHERE ent_id=$ent_id", ARRAY_N );

		return $data;
	}

	/**
	 * Advanced Page Visit Counter refresh dashboard charts premium users only.
	 *
	 * @since    4.4.0
	 */
	public function apvc_refresh_dashboard_chart__premium_only() {

		if ( ! wp_verify_nonce( $_POST['security'], 'security_nonce' ) ) {
			die( 'Permission Denied.' );
		}

		global $wpdb;
		$table = APVC_DATA_TABLE;

		$duration = sanitize_text_field( $_REQUEST["duration"] );

		if ( $duration !== "" ) {

			$totalDays = 0;
			if ( $duration == "7_days" ) {
				$totalDays = 7;
			} else if ( $duration == "1_month" ) {
				$totalDays = 30;
			} else if ( $duration == "3_months" ) {
				$totalDays = 90;
			} else if ( $duration == "6_month" ) {
				$totalDays = 180;
			}

			$labels = [];
			$dCount = [];
			$vCount = [];

			for ( $i = 0; $i < $totalDays; $i ++ ) {

				$iTemp = $i;
				if ( $duration == "6_month" ) {
					$i += 6;
				}
				$labels[] = date( "d-M", strtotime( '-' . $i . ' days' ) );
				$sDate    = date( "Y-m-d 0:0:0", strtotime( '-' . $i . ' days' ) );
				$eDate    = date( "Y-m-d 23:59:59", strtotime( '-' . $i . ' days' ) );

				$dCount[] = $wpdb->get_var( "SELECT COUNT(*) FROM {$table} WHERE date >= '$sDate' AND date <= '$eDate'" );

				$vCount[] = $wpdb->get_var( "SELECT COUNT(*) FROM {$table} WHERE date >= '$sDate' AND date <= '$eDate' GROUP BY ip_address" );
				$i        = $iTemp;
			}

			echo wp_json_encode( [
				"message" => "Success",
				"chart"   => json_encode( [
					"labels"   => $labels,
					"visitors" => $vCount,
					"visits"   => $dCount,
				] ),
			] );

		}

		wp_die();

	}


	public function apvc_get_chart_data_single() {

		if ( ! wp_verify_nonce( $_POST['security'], 'security_nonce' ) ) {
			die( 'Permission Denied.' );
		}

		global $wpdb;
		$table = APVC_DATA_TABLE;

		$article  = sanitize_text_field( $_REQUEST["article"] );
		$days     = sanitize_text_field( $_REQUEST["days"] );
		$labels   = [];
		$dCount   = [];
		$vCount   = [];
		$andQuery = "";
		if ( $days == 0 || $days == "" ) {
			$days = 20;
		}

		if ( apvc_fs()->is__premium_only() ) {

			$d = sanitize_text_field( $_REQUEST['sdate'] );
			$e = sanitize_text_field( $_REQUEST['edate'] );

			$user_type = sanitize_text_field( $_REQUEST['user_type'] );
			$cnt       = implode( "','", sanitize_text_field( $_REQUEST['country'] ) );
			$dv        = implode( "','", sanitize_text_field( $_REQUEST['devices'] ) );
			$br        = implode( "','", sanitize_text_field( $_REQUEST['browser'] ) );
			$os        = implode( "','", sanitize_text_field( $_REQUEST['os'] ) );

			if ( $d != 0 && ! $this->apvc_is_date( $d ) ) {
				$days_ago = date( 'Y-m-d', time() - (int)$d * (int)APVC_SECONDS_PER_DAY );
				$andQuery .= " AND date >= '$days_ago' ";
			}

			if ( $this->apvc_is_date( $d ) && $this->apvc_is_date( $e ) ) {
				$andQuery .= " AND date >= '$d' AND date <= '$e' ";
			}

			if ( $cnt != '' ) {
				$andQuery .= " AND country IN ('$cnt') ";
			}

			if ( $dv != '' ) {
				$andQuery .= " AND device_type IN ('$dv') ";
			}

			if ( $br != '' ) {
				$andQuery .= " AND browser_short_name IN ('$br') ";
			}

			if ( $os != '' ) {
				$andQuery .= " AND operating_system IN ('$os') ";
			}

			if ( $user_type == 'registered' ) {
				$andQuery .= " AND user_type = 'Registered' ";
			} else {
				$andQuery .= " AND user_type = 'Guest' ";
			}

			for ( $i = 0; $i < $days; $i ++ ) {

				$labels[] = date( "d-M", strtotime( '-' . $i . ' days' ) );
				$sDate    = date( "Y-m-d 0:0:0", strtotime( '-' . $i . ' days' ) );
				$eDate    = date( "Y-m-d 23:59:59", strtotime( '-' . $i . ' days' ) );

				$dCount[] = $wpdb->get_var( "SELECT COUNT(*) FROM {$table} WHERE date >= '$sDate' AND date <= '$eDate' AND article_id= $article $andQuery " );

				foreach ( $dCount as $key => $value ) {
					if ( $value === NULL ) {
						$dCount[ $key ] = 0;
					}
				}

				$vCount[] = $wpdb->get_var( "SELECT COUNT(*) FROM {$table} WHERE date >= '$sDate' AND date <= '$eDate' AND article_id=$article $andQuery GROUP BY ip_address" );

				foreach ( $vCount as $key => $value ) {
					if ( $value === NULL ) {
						$vCount[ $key ] = 0;
					}
				}
			}
			echo wp_json_encode( [
				"message" => "Success",
				"chart"   => json_encode( [
					"labels"   => $labels,
					"visitors" => $vCount,
					"visits"   => $dCount,
				] ),
			] );

		} else {

			for ( $i = 0; $i < 20; $i ++ ) {
				$labels[] = date( "d-M", strtotime( '-' . $i . ' days' ) );
				$sDate    = date( "Y-m-d 0:0:0", strtotime( '-' . $i . ' days' ) );
				$eDate    = date( "Y-m-d 23:59:59", strtotime( '-' . $i . ' days' ) );

				$dCount[] = $wpdb->get_var( "SELECT COUNT(*) FROM {$table} WHERE date >= '$sDate' AND date <= '$eDate' AND article_id='$article' '$andQuery' " );

				$vCount[] = $wpdb->get_var( "SELECT COUNT(*) FROM {$table} WHERE date >= '$sDate' AND date <= '$eDate' AND article_id='$article' GROUP BY ip_address" );
			}
			echo wp_json_encode( [
				"message" => "Success",
				"chart"   => json_encode( [
					"labels"   => $labels,
					"visitors" => $vCount,
					"visits"   => $dCount,
				] ),
			] );

		}

		wp_die();
	}

}



